"""
This module is a main module. It provides a Protocol Buffers compiler for Java.
The resulting Java code is much simpler and easier to use than the Java code
that Google's protoc generates. Message classes are mutable, which makes things
a ton easier to use, but they also have setters that return the message, which
allows for chaining (new SomeMessageType().setX(...).setY(...).setZ(...)).

Right now, this only supports really basic protobuf functionality. It does not
check a good majority of stuff that protoc generally should, instead relying on
the Java compiler to do this. It does not require any external libraries to be
on the classpath; the generated code contains everything needed to perform
encoding. (It heavily relies on DataInputStream and DataOutputStream for
encoding and decoding functions.)

Multiple messages in a single file are supported (and they can reference each
other), but nested messages (messages defined within messages) are not. Enums
are not yet supported. Required, optional, and repeated fields are all fully
supported. Defaults are not; fields in the generated classes are stored as
their Java wrapper types, and if they're not present in the message, they'll be
null (unless they're marked as required, in which case the class will throw an
exception when trying to deserialize the message). Unknown fields are not
allowed; if an unknown field is encountered while deserializing, an exception
will be thrown. If someone wants to add support for unknown fields, I'd greatly
appreciate it.

Optional and required fields' classes are their wrapper type or the generated
class of the message for a compound field. Repeated fields are lists of the
corresponding types for that entry. These fields are initialized to contain
ArrayList instances, but any list instance is fine for the serializer; it's
perfectly fine for some user of the class to replace the list with a
LinkedList, for example, if they see fit.

The java_package and java_outer_classname options must be specified in the
protobuf file, and they must be specified before any other messages. All
options, for that matter, must be specified before messages are defined. This
is a limitation I might remove at some point.

The supported primitive types are string, int32, int64, double, and bool.

Only single-line comments (I.E. those starting with "//") are supported.

This module uses pyparsing to parse the protobuf file. You'll need pyparsing
installed in order for this module to work. 
"""

from pyparsing import Literal, Regex, ZeroOrMore, Forward, Optional, Group
from pyparsing import Keyword, Suppress, LineEnd
import sys
import os
import re

class Field(object):
    def __init__(self, tokens):
        tokens = tokens.asList()
        self.mode = tokens[0] # required, optional, repeated
        self.type = tokens[1] # int32, bool, string, etc
        self.name = tokens[2] # name of the field
        self.number = int(tokens[3]) # field index in the protocol
    
    def get_java_type(self):
        type = java_type_map.get(self.type, self.type)
        if self.mode == "repeated":
            type = "List<" + type + ">"
        return type
    
    def get_java_default(self):
        if self.mode == "repeated":
            return "new ArrayList<" + java_type_map.get(self.type, self.type) + ">()"
        return "null"
    
    def __repr__(self):
        return ("<Field " + self.name + " of type " + self.type + ", " + 
                self.mode + ", #" + str(self.number) + ">")
    __str__ = __repr__

class Message(object):
    def __init__(self, tokens):
        tokens = tokens.asList()
        self.name = tokens[0]
        self.fields = tokens[1]
    
    def __repr__(self):
        return "<Message " + self.name + " with fields " + repr(self.fields) + ">"
    __str__ = __repr__

comment = Literal("//") + Regex("[^\\n]*") + LineEnd()
field_type = Regex("[A-Za-z0-9_]+")
field_name = Regex("[A-Za-z0-9_]+")
field_number = Regex("[0-9]+")
message_field = ((Keyword("required") | Keyword("optional") | Keyword("repeated")) 
        + field_type + field_name + Suppress("=") + field_number + Suppress(";"))
message_field.setParseAction(Field)
message_name = Regex("[A-Za-z_]+")
message_spec = (Suppress(Keyword("message")) + message_name + Suppress("{") + 
         Group(ZeroOrMore(message_field)) + Suppress("}"))
message_spec.setParseAction(Message)
option_spec = (Suppress(Keyword("option")) + Regex("[a-z_]+") + Suppress("=") + 
        Regex('"[^"]*"') + Suppress(";"))
option_spec.setParseAction(lambda tokens: (tokens[0], tokens[1][1:-1]))
option_list = ZeroOrMore(option_spec)
option_list.setParseAction(lambda tokens: dict(tokens.asList()))
message_list = Group(ZeroOrMore(message_spec))
proto_file = (option_list + message_list).ignore(comment)

if len(sys.argv) < 3:
    print "usage: simpleproto some_file.proto outputfolder"
    print "The output will be placed in a folder within outputfolder"
    print "appropriate for the package specified in the file's java_package"
    print "option, in the file specified by java_outer_classname."

options, messages = proto_file.parseFile(sys.argv[1], parseAll=True).asList()
output_class = options["java_outer_classname"]
output_package = options["java_package"]
output_file_path = os.path.join(sys.argv[2], output_package.replace(
        ".", os.path.sep), output_class + ".java")
output_file = open(output_file_path, "w")

default_printf_args = {"output_class": output_class, "output_package":
        output_package}

# Maps protobuf types to the corresponding Java field types
java_type_map = {"string": "String", "int32": "Integer", "int64": "Long",
        "double": "Double", "bool": "Boolean"}
# Maps protobuf types to their expected wire type
wire_type_map = {"string": 2, "int32": 0, "int64": 0, "double": 1, "bool": 0}

def write(text, **kwargs):
    args = {}
    args.update(default_printf_args)
    args.update(kwargs)
    result = text % args
    result = re.sub("[\n]+", "\n", result)
    while result.startswith("\n"):
        result = result[1:]
    output_file.write(result)

write("""
package %(output_package)s;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.List;
import java.util.ArrayList;

public class %(output_class)s {
public static interface GeneratedMessage {public void serialize(DataOutputStream
out);public void deserialize(DataInputStream in);}

protected static byte[] reverseBytes(byte[] in){
    byte[] out = new byte[in.length];
    for(int i = 0; i < in.length; i++){
        out[(in.length-i)-1] = in[i];
    }
    return out;
}
protected static byte[] readVarint(DataInputStream in){
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    int value = in.read();
    while((value&0x80)>0){
        out.write(value);
        value = in.read();
    }
    out.write(value);
    return out.toByteArray()
}
protected static long readVarintAsLong(DataInputStream in){
    long result = 0;
    for(byte value : reverseBytes(readVarint(in))){
        result <<= 7;
        result = result | (value & 0x7F);
    }
    return result;
}
protected static int readVarintAsInt(DataInputStream in){
    return (int) readVarintAsLong(in);
}
protected static void writeVarintAsLong(DataOutputStream out, long value){
    for(int i = 0; i < 10; i++){
        out.write((value&0x7F)|0x80);
        out >>= 
    }
} 
""")

for message in messages:
    write("""
    public static class %(name)s implements GeneratedMessage { 
    """, name=message.name)
    for field in message.fields:
        write("""
        public %(type)s %(name)s = %(default)s;
        """, type=field.get_java_type(), name=field.name, default=field.get_java_default())
    write("public void serialize(DataOutputStream out){}public void deserialize(DataInputStream in){}")
    write("}")

write("}\n")

output_file.close()


































RegisterInterfaceCommand = "register_interface_command"
RegisterInterfaceResponse = "register_interface_response"
RegisterFunctionCommand = "register_function_command"
RegisterFunctionResponse = "register_function_response"
CallFunctionCommand = "call_function_command"
CallFunctionResponse = "call_function_response"
RunFunctionCommand = "run_function_command"
RunFunctionResponse = "run_function_response"
ErrorResponse = "error_response"
RegisterEventCommand = "register_event_command"
RegisterEventResponse = "register_event_response"
RegisterListenerCommand = "register_listener_command"
RegisterListenerResponse = "register_listener_response"
DeregisterListenerCommand = "deregister_listener_command"
DeregisterListenerResponse = "deregister_listener_response"
FireEventCommand = "fire_event_command"
RegisterObjectCommand = "register_object_command"
RegisterObjectResponse = "register_object_response"
WatchObjectCommand = "watch_object_command"
WatchObjectResponse = "watch_object_response"
UnwatchObjectCommand = "unwatch_object_command"
UnwatchObjectResponse = "unwatch_object_response"
SetObjectCommand = "set_object_command"
GetObjectValueCommand = "get_object_value_command"
GetObjectValueResponse = "get_object_value_response"
PingCommand = "ping_command"
PingResponse = "ping_response"
ActivateHackCommand = "activate_hack_command"
ActivateHackResponse = "activate_hack_response"




from BeautifulSoup import BeautifulSoup
import urllib
import urllib2
from getpass import getpass
import re
from decimal import Decimal

def main():
    print "Testing out the University Federal Credit Union client..."


class Session(object):
    # Thanks to http://stackoverflow.com/a/4836113/992720 for the general idea
    # of how to track cookies and redirects
    def __init__(self, username, password=None, login=True):
        if password is None:
            password = getpass("Enter your password: ")
        self.username = username
        self.password = password
        cookies = urllib2.HTTPCookieProcessor()
        redirector = urllib2.HTTPRedirectHandler()
        self._opener = urllib2.build_opener(redirector, cookies)
        if login:
            self.login()
    
    def _get(self, url, parameters=None):
        return self._opener.open(url + ("?" + urllib.urlencode(parameters) if parameters else "")).read()
    
    def _post(self, url, parameters):
        return self._opener.open(url, urllib.urlencode(parameters)).read()
    
    def login(self):
        main_page = BeautifulSoup(self._get("https://mobile.ucreditu.com/"))
        login_url = main_page.find("a", {"name": "LogIn"})["href"]
        login_page = BeautifulSoup(self._get("https://mobile.ucreditu.com/" + login_url[1:]))
        login_submit_url = login_page.find("form", {"name": "signin"})["action"]
        summary_page = self._post("https://mobile.ucreditu.com/" + login_submit_url[1:],
                {"userNumber": self.username, "password": self.password, "Login": "Login"})
        try:
            checking_location = summary_page.find("E-Checking")
            available_location = summary_page.find("Available", checking_location)
            close_b_location = summary_page.find("</b>", available_location)
            open_br_location = summary_page.find("<br>", close_b_location)
            available = summary_page[close_b_location+len("</b>"):open_br_location]
            available = Decimal(available)
            self.checking_balance = available
        except Exception as e:
            # TODO: handle this in some manner
            raise


































from functools import partial
import json

__doc__ = """\
This module implements a sort of serialization mechanism. It uses JSON as the
underlying encoding. It requires classes that are to be encoded to be decorated
with @serializable, which requires some information about which fields and
stuff can be serialized and what types they map to. The idea is that the
serialization scheme provided by this module is cross-language so that objects
serialized in Python could, for example, be deserialized in Java and vice
versa. To that end, the serializer is essentially statically-typed, although it
takes care of enforcing type constraints for you so you don't have to do it
yourself.

This module does not handle reference cycles. If you have any, you'll put the
serializer into an infinite loop. This doesn't pose any vulnerabilites to the
deserializer or anything, since the protocol itself doesn't support this.

Subclasses of built-in Python types (such as int or str) must be hashable for
the serializer to work correctly. They will generally get deserialized to their
Python built-in equivalent.

Classes have to be registered with a serializer instance before instances of
them can be serialized or deserialized. This is so that the serializer can use
language-independent names for objects.

Currently, this module only works properly with new-style classes.
"""

_NoValue = object()

class SerializerException(Exception):
    """
    Instances of this type or instances of subclasses thereof are raised when
    serialization problems happen.
    """

class NullException(SerializerException):
    """
    Raised when an object being serialized or deserialized contains a field
    that is null when the class's serial_info specified that the field cannot
    be null.
    """
    pass

class TypeMismatchException(SerializerException):
    """
    Raised when an object being serialized or deserialized contains a field
    whose value is not one of the allowed types for that field.
    """
    pass

class Serializer(object):
    """
    The main class in the serialization module. Instances of this class are
    used for serialization and deserialization. Once an instance has been
    configured, it is thread-safe for concurrent serialization and
    deserialization.
    """
    def __init__(self):
        self.types_to_names = {}
        self.names_to_types = {}
    
    def serializable(self, name, target_class=None):
        """
        Registers a class under the specified name for use with this
        serializer. The target class is returned. If the target class is not 
        specified, this function returns a function that can be passed the 
        target class with the same effect as calling this function itself,
        which allows this function to be used as a decorator thus:
        
        @some_serializer.serializable("the_class_name")
        class SomeClass(object):
            ...
        
        This is the reason this function returns the target class.
        Additionally, a class can be specified in place of the name, without
        specifying any value for target_class, which will result in this
        function acting as if it had been called with the specified class's
        name and the specified class passed as arguments. This allows this
        function to be used as a more compact decorator when the name is the
        same as the class's name thus:
        
        @some_serializer.serializable
        class SomeClass(object):
            ...
        
        which would register the class under the name "SomeClass".
        
        The target class should have an attribute named serial_info. This
        function will raise an exception if the attribute is missing or not
        well-formed. The attribute should be a list of sequences. Each sequence
        should have three items: the name of a field on the object that is to
        be serialized, a type or string or a sequence of types and strings representing
        the types that can be stored into the specified field, and a boolean
        indicating whether or not the field can be None. The second list, the
        one that specifies types and strings, consists of types for the
        primitive types (such as int, str, float, etc.) and strings for any
        type registered as serializable. Types specified thus as names are not
        checked for validity in case they have not yet been registered as
        serializable.
        """
        if isinstance(name, type):
            return self.serializable(name.__name__, name)
        if target_class is None:
            return partial(self.serializable, name)
        self.types_to_names[target_class] = name
        self.names_to_types[name] = target_class
        serial_info = target_class.serial_info
        serial_info_map = {}
        target_class.serial_info_map = serial_info_map
        for name, type_list, nullable in serial_info:
            if isinstance(type_list, (type, str)): 
                type_list = tuple([type_list])
            elif isinstance(type_list, list):
                type_list = tuple(type_list)
            serial_info_map[name] = type_list, nullable
        return target_class
    
    def serialize(self, serial_object):
        representation = self._serialize_to_object(serial_object)
        return json.dumps(representation, separators=(",",":"))
    
    def _serialize_to_object(self, serial_object):
        if isinstance(serial_object, (int, long, float, str, dict, type(None))):
            return serial_object
        if isinstance(serial_object, (list, tuple)):
            return [None, serial_object]
        # Not a primitive type, so we'll serialize it as an object.
        object_type = type(serial_object)
        info = object_type.serial_info_map
        result_map = {}
        result_list = [self.types_to_names[object_type], result_map]
        for name in info:
            type_list, nullable = info[name]
            try:
                value = getattr(serial_object, name)
            except:
                value = _NoValue
            if (not nullable) and value is None:
                raise NullException("Instance of " + str(object_type) + 
                                    ", attribute " + name)
            if value is not _NoValue:
                required_types = self._get_required_types(type_list)
                if not isinstance(value, required_types + (type(None),)):
                    raise TypeMismatchException("Expected object of type " + 
                                str(list(required_types)) + " on instance of "
                                + str(object_type) + " but got object of type "
                                + str(type(value)) + " instead")
                result_map[name] = self._serialize_to_object(value)
        return result_list
    
    def _get_required_types(self, type_list):
        return tuple((self.names_to_types[type_name] if 
                     isinstance(type_name, str) else type_name) for type_name
                     in type_list)
    
    def deserialize(self, text):
        model = json.loads(text)
        return self._deserialize_from_model(model)
    
    def _deserialize_from_model(self, model):
        if isinstance(model, (int, long, float, str, dict, type(None))):
            return model
        if isinstance(model, unicode):
            return str(model) # Need to add real support for unicode in the
            # future, maybe consider coercing str and unicode as specified
            # on an attribute's required values or something
        if not isinstance(model, list):
            raise SerializerException("Invalid instance type " + 
                                      str(type(model)) + " received")
        if model[0] is None: # Encoded list
            return model[1]
        # Encoded instance
        object_type = self.names_to_types[model[0]]
        info = object_type.serial_info_map
        result = object_type()
        object_map = model[1]
        for name in info:
            type_list, nullable = info[name]
            if name in object_map:
                setattr(result, name, 
                        self._deserialize_from_model(object_map[name]))
            try:
                value = getattr(result, name)
            except:
                value = _NoValue
            if (not nullable) and (value is None or value is _NoValue):
                raise NullException("Instance of " + str(object_type) + 
                                    ", attribute " + name)
            if value is not _NoValue:
                required_types = self._get_required_types(type_list)
                if not isinstance(value, required_types + (type(None),)):
                    raise TypeMismatchException("Expected object of type " + 
                                str(list(required_types)) + " on instance of "
                                + str(object_type) + " but got object of type "
                                + str(type(value)) + " instead")
        return result






















from traceback import print_exc

class Event(object):
    """
    An event manager. Listeners can be added by calling the listen function.
    Listeners can be removed by calling the unlisten function. The replace
    function removes all listeners and then adds one specific listener, in
    essence replacing all of the current listeners with the one specified.
    
    Calling an instance of this class fires the event. All arguments passed
    to the invocation will be passed to each listener. If any of the
    listeners raise an exception, the exception's traceback will be printed
    to stdout, but the exception will be otherwise suppressed (and in
    particular, all remaining listeners will still be called).
    """
    def __init__(self):
        """
        Creates a new event.
        """
        self.listeners = []
    
    def listen(self, function):
        """
        Adds the specified function as a listener to this event. If the
        function is already present on this event, it will not be added twice.
        """
        if function not in self.listeners:
            self.listeners.append(function)
    
    def unlisten(self, function):
        """
        Removes the specified function from this event's list of listeners.
        """
        try:
            self.listeners.remove(function)
        except ValueError:
            pass
    
    def __call__(self, *args, **kwargs):
        """
        Fires all listeners registered to this event. The specified arguments
        (including keyword arguments) will be passed to all of the listeners.
        """
        for listener in self.listeners[:]: # Clone in case someone modifies us
            try:
                listener(*args, **kwargs)
            except:
                print_exc()




class Singleton(object):
    def __init__(self, name):
        self.name = name
        self.short_name = name.rpartition(".")[2]
    
    def __str__(self):
        return self.name
    
    def __repr__(self):
        return self.name




from libautobus import AutobusConnection
from traceback import print_exc
import sys
import os
from time import sleep
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
from urlparse import parse_qsl
from time import time
from concurrent import synchronized
try:
    from win32com.client import Dispatch #@UnresolvedImport
    import pythoncom
except ImportError:
    print_exc()
    print "You need to be on a Windows machine to use activehomed, and you"
    print "need to have pywin32 installed."
    sys.exit()

port = 53306
RF_DELAY = 0.33
PLC_DELAY = 0.25
last_plc_time = 0.0
last_plc_address = "a1"
last_rf_time = 0.0
last_rf_address = "a1"

class HTTPHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        global last_plc_time
        global last_plc_address
        global last_rf_time
        global last_rf_address
        path = self.path
        if "?" not in path: # Serve the script page
            print "Serving script page for url " + path
            self.send_response(200)
            self.send_header("Content-Type", "text/html")
            self.end_headers()
            with open("src/activehomed/receive_ie.html") as file:
                for line in file:
                    self.wfile.write(line + " ")
        else: # Incoming event from the script page
            print "Inbound event for url " + path
            params = dict(parse_qsl(path.split("?")[1], True))
            mode, address, command, sequence, timestamp, _, _ = (params["a"],
                    params["b"], params["c"], params["d"],
                    params["e"], params["f"], params["g"])
            receive_action_event(mode, address, command, sequence, timestamp)
            repeat = False
            if mode.startswith("send") or mode.startswith("recv"):
                mode = mode[4:]
            now = time()
            if mode == "plc":
                # Command is a repeat if it's within PLC_DELAY seconds of the
                # last command and it's either for the same address as the
                # previous command or the command is one of the
                # address-agnostic commands such as AllLightsOn
                repeat = ((address == last_plc_address
                            or command in ["AllLightsOn", "AllUnitsOff", "AllLightsOff"])
                        and (last_plc_time + PLC_DELAY) > now)
                last_plc_time = now
                last_plc_address = address
            elif mode == "rf":
                repeat = (address == last_rf_address
                        and (last_rf_time + RF_DELAY) > now)
                last_rf_time = now
                last_rf_address = address
            house, unit = address[0], address[1:]
            if sequence == "":
                sequence = None
            else:
                try:
                    sequence = int(sequence)
                except ValueError:
                    print ("WARNING: Sequence number " + str(sequence) +
                            " couldn't be converted to an integer.")
            if timestamp == "":
                timestamp = None 
            receive_event(mode, house, unit, address, command, sequence,
                    timestamp, repeat)
            self.send_response(200)
            self.send_header("Content-Type", "text/plain")
            self.end_headers()
            self.wfile.write("ok")

class RPC(object):
    """
    This interface allows for access to an X10 CM15A module. It allows two-way
    access: powerline and RF commands can be sent, and powerline and RF
    commands can be received. Command lines can be sent with the same syntax
    as the ActiveHome scripting SDK's SendAction method by using this
    interface's action function. Inbound powerline and RF events can be
    detected by registering a listener on this interface's receive and
    receive_action events. (The former should be preferred; it detects repeated
    commands sent by holding down a controller button among other things,
    while the latter is essentially equivalent in signature to the ActiveHome
    scripting SDK's OnRecvAction event, but without the last two parameters
    since the scripting SDK doesn't yet use those.)
    
    One random note: I've noticed that the CM15A seems to receive all powerline
    commands that it sends out, but it doesn't seem to receive any of the RF
    commands that it sends out. Be aware, therefore, that if you send a
    powerline command, the receive and receive_action events will be fired for
    that command, but sending an RF command won't cause the events to be fired.
    """
    def action(self, mode, *args):
        """
        Sends an X10 action. This function has an almost identical signature to
        the SendAction function provided by the ActiveHome scripting SDK's
        ActiveX object. To make it easier on autosend users, all of
        the remaining arguments after the mode (which should be either sendplc
        or sendrf; queryplc should work but I haven't tested it yet) will be
        concatenated with space characters inbetween and the result used as the
        action to perform.
        
        The return value of this function is normally 0. It's most likely going
        to be different if the mode is queryplc or something other than
        sendplc or sendrf; I haven't looked up their documentation yet, though,
        so I don't know precisely what the return values are for those modes.
        """
        pythoncom.CoInitialize()
        try:
            args = " ".join(str(arg) for arg in args)
            return Dispatch("X10.ActiveHome").SendAction(mode, args)
        finally:
            pythoncom.CoUninitialize()

def main():
    global bus
    global receive_event
    global receive_action_event
    pythoncom.CoInitialize()
    try:
        Dispatch("X10.ActiveHome") # Make sure we've got it before hand
    except:
        print_exc()
        print "You need to install the ActiveHome Scripting SDK before you"
        print "can use activehomed."
        sys.exit()
    print "Connecting to server " + str(os.getenv("AUTOBUS_SERVER"))
    bus = AutobusConnection()
    bus.add_interface("activehome", RPC())
    receive_action_event = bus.add_event("activehome", "receive_action", """\
    Fired when the CM15A receives a powerline or RF signal. This event is quite
    low-level and hence the receive event should be preferred to this one. The
    signature of this event is identical to the ActiveHome Scripting SDK's
    OnRecvAction event, except that all parameters are passed as strings and
    the last two parameters are not present (the scripting SDK doesn't make
    use of those at present, so you're not losing any information).
    """)
    receive_event = bus.add_event("activehome", "receive", """\
    receive(mode, house, unit, address, command, sequence, timestamp, repeat)
    
    Fired when the CM15A receives a powerline or RF signal. This event is more
    high-level than receive_action.
    
    mode is the mode, which will either be plc or rf.
    
    house is the house code. unit is the unit code. address is the string
    concatenation of the house and unit codes. For example, house, unit, and
    address could be "d", "2", and "d2", respectively. All three parameters are
    passed as strings.
    
    command is the command that occurred. The ActiveHome Scripting SDK contains
    a list of all valid commands, but some of the more common ones are On, Off,
    AllLightsOn, AllUnitsOff, Bright, Dim, PanLeft, CamGoPosition1, and
    DeviceStop (the latter of which occurs when the CM15A is unplugged from the
    computer).
    
    sequence is slightly command-specific, as follows:
    
    If this is an RF command, sequence will be the sequence number, which
    is 0 for the button being pressed down, 1 and upward for holding
    the button down (fired roughly six times per second), and -1 for
    letting go of the button. A word of warning: this doesn't seem to
    be hugely reliable. For example, pushing an on button on a ScanPad
    or PalmPad results in six events: three pairs of button press and button 
    release commands, despite the fact that the button was only pressed
    once. Holding the button down just repeats these pairs further. If you want
    a reliable indicator of whether or not a button press is a repeat, use the
    repeat parameter passed to this event.
    
    If this is a powerline command, sequence will be null unless the command is
    Bright or Dim, in which case it will be the bright/dim level. When a
    bright/dim button is held down on an RF controller, the result is a
    repeated firing of Bright/Dim commands. When a bright/dim button is held
    down on a powerline controller, the result is that no events are fired
    until the button is let go, and then a single Bright/Dim command is fired
    with the sequence parameter being set to the percent change in brightness
    that would have resulted had the command been directed at an actual module.
    
    The timestamp parameter is only present for RF commands. It specifies, as
    a string, the date and time at which the command was received. This
    generally won't be used since Autobus only incurs a few milliseconds'
    latency, so the date could just be detected locally.
    
    repeat is a boolean. It will be set to true if this event is most likely a
    repeat of another event. For example, if an on button on a ScanPad
    controller is pressed, the result is three pairs of button press and button
    release commands. The first command will have its repeat parameter set to
    false, and the remaining five commands will have their repeat parameters
    set to true. Another example is a bug in the CM15A: If two units were
    simultaneously turned on (for example, by A1 A2 A off going across the
    powerline), and then an AllLightsOn received on the powerline for that
    house code, the CM15A will report that as two events: A1 AllLightsOff and
    A2 AllLightsOff. The latter will be detected as a repeat when this event
    is fired. NOTE: There are currently problems with detecting RF
    bright/dim repeats, so you should generally ignore this parameter for
    RF bright/dim commands.
    """)
    bus.start_connecting()
    try:
        HTTPServer(("127.0.0.1", port), HTTPHandler).serve_forever()
    except KeyboardInterrupt:
        print "Interrupted, shutting down"
    finally:
        bus.shutdown()
    


























from libautobus import create_message, COMMAND, RESPONSE
from libautobus import NOTIFICATION, encode_object
import autobus
from libautobus.message_types import *

def process_register_interface_command(message, sender, connection):
    name = message["name"]
    doc = message["doc"]
    print "Registering interface with name " + name
    interface = autobus.Interface(name, connection, doc)
    try:
        interface.register()
    except KeyError:
        connection.send_error(message, text="An interface with the same name "
                "is already registered.")
        return
    response = create_message(RegisterInterfaceResponse, message)
    connection.send(response)

def process_register_function_command(message, sender, connection):
    interface_name = message["interface_name"]
    function_names = message["name"]
    docs = message["doc"]
    if isinstance(function_names, basestring) and isinstance(docs, basestring):
        function_names = [function_names]
        docs = [docs]
    try:
        interface = autobus.lookup_interface(interface_name, sender)
    except autobus.NoSuchInterfaceException as e:
        connection.send_error(message, text=str(e))
        return
    try:
        for function_name, doc in zip(function_names, docs):
            interface.register_function(sender, function_name, doc, notify=False)
    except KeyError:
        connection.send_error(message, "A function with that name has already "
                "been registered on that interface.")
        autobus.autobus_interface.notify_object("interface_items")
        return
    autobus.autobus_interface.notify_object("interface_items")
    response = create_message(RegisterFunctionResponse, message)
    connection.send(response)

def process_call_function_command(message, sender, connection):
    interface_name = message["interface_name"]
    function_name = message["function"]
    print ("Attempting to dispatch function call from " + str(sender) + 
            " to interface " + interface_name + " and function " + function_name)
    try:
        interface = autobus.lookup_interface(interface_name)
        function = interface.lookup_function(function_name)
    except (autobus.NoSuchInterfaceException, autobus.NoSuchFunctionException) as e:
        print "No such interface. Dispatch failed."
        connection.send_error(message, text=str(e))
        return
    if function.special:
        print "Dispatching as special function call"
        function.invoke_special(message, connection)
        return
    print "Dispatching as normal function call"
    invoke_message = create_message(RunFunctionCommand,
            interface_name=interface.name, function=function.name,
            arguments=message["arguments"])
    interface.connection.send(invoke_message)
    print ("Sending run command to " + str(interface.connection.id) + 
            " with message id " + str(invoke_message["message_id"]) + 
            " whose response is to be forwarded with id " + 
            str(message["message_id"]))
    # FIXME: This causes a response to a function invocation sent as a notification
    # to still be sent when the function returns on the remote side. We need to
    # somehow not add the message to this list and perhaps suppress the "sporadic
    # message received" warning that would consequently be printed to stdout once
    # the remote client sends back the return value if the incoming message here
    # is a notification.
    interface.connection.pending_responses[invoke_message["message_id"]] = (
            sender, message["message_id"])

def process_run_function_response(message, sender, connection):
    response = create_message(CallFunctionResponse,
            RESPONSE, return_value=message["return_value"])
    if not message["message_id"] in connection.pending_responses:
        # Sporadic response received, just ignore it
        print ("Sporadic run response received from connection " + str(sender)
                + " for reported message id " + str(message["message_id"]) + 
                ". It will be ignored.")
        return
    response_connection_id, response_message_id = connection.pending_responses[message["message_id"]]
    del connection.pending_responses[message["message_id"]]
    response["message_id"] = response_message_id
    if response_connection_id in autobus.connection_map: # Make sure the connection
        # listening for the response is still alive
        response_connection = autobus.connection_map[response_connection_id]
        response_connection.send(response)

def process_register_event_command(message, sender, connection):
    interface_name = message["interface_name"]
    event_name = message["event_name"]
    doc = message["doc"]
    try:
        interface = autobus.lookup_interface(interface_name, sender)
    except autobus.NoSuchInterfaceException as e:
        connection.send_error(message, text=str(e))
        return
    try:
        interface.create_and_register_event(sender, event_name, doc)
    except KeyError:
        connection.send_error(message, "An event with that name has already "
                "been registered on that interface.")
        return
    response = create_message(RegisterEventResponse, message)
    connection.send(response)

def process_register_listener_command(message, sender, connection):
    interface_name = message["interface_name"]
    event_name = message["event_name"]
    autobus.register_event_listener(interface_name, event_name, sender)
    connection.listeners.append((interface_name, event_name))
    response = create_message(RegisterListenerResponse,
            message, interface_name=interface_name, event_name=event_name)
    connection.send(response)

def process_fire_event_command(message, sender, connection):
    interface_name = message["interface_name"]
    event_name = message["event_name"]
    arguments = message["arguments"]
    if arguments.__class__ is not list:
        connection.send_error(message, text="Event arguments must be "
                "specified as a list")
        return
    try:
        interface = autobus.lookup_interface(interface_name, sender)
    except autobus.NoSuchInterfaceException as e:
        connection.send_error(message, text=str(e))
        return
    if event_name not in interface.event_map:
        connection.send_error(message, "You need to register the event "
                "with Autobus before you can fire it.")
        return
    interface.event_map[event_name].notify(arguments)

def process_register_object_command(message, sender, connection):
    interface_name = message["interface_name"]
    object_name = message["object_name"]
    doc = message["doc"]
    value = message["value"]
    try:
        interface = autobus.lookup_interface(interface_name, sender)
    except autobus.NoSuchInterfaceException as e:
        connection.send_error(message, text=str(e))
        return
    try:
        interface.create_and_register_object(sender, object_name, doc, value)
    except KeyError:
        connection.send_error(message, "An object with that name has already "
                "been registered on that interface.")
        return
    response = create_message(RegisterObjectResponse, message)
    connection.send(response)

def process_watch_object_command(message, sender, connection):
    interface_name = message["interface_name"]
    object_name = message["object_name"]
    print ("Registering object watch from " + str(sender) + 
            " on interface " + interface_name + " object " + object_name)
    autobus.register_object_watch(interface_name, object_name, sender)
    connection.watches.append((interface_name, object_name))
    try:
        interface = autobus.lookup_interface(interface_name)
        object = interface.object_map[object_name]
        object_value = object.get()
    except (autobus.NoSuchInterfaceException, KeyError): # Object doesn't exist yet
        object_value = encode_object(None)
    response = create_message(WatchObjectResponse,
            message, interface_name=interface_name, object_name=object_name,
            value=object_value)
    connection.send(response)

def process_set_object_command(message, sender, connection):
    interface_name = message["interface_name"]
    object_name = message["object_name"]
    value = message["value"]
    try:
        interface = autobus.lookup_interface(interface_name, sender)
    except autobus.NoSuchInterfaceException as e:
        connection.send_error(message, text=str(e))
        return
    if object_name not in interface.object_map:
        connection.send_error(message, "You need to register the object "
                "with Autobus before you can set its value.")
        return
    interface.object_map[object_name].set_and_notify(value)

def process_ping_command(message, sender, connection):
    connection.send_new(PingResponse, message)

def process_get_object_value_command(message, sender, connection):
    interface_name = message["interface_name"]
    object_name = message["object_name"]
    try:
        interface = autobus.lookup_interface(interface_name)
        object = interface.object_map[object_name]
        object_value = object.get()
    except (autobus.NoSuchInterfaceException, KeyError): # Object doesn't exist yet
        object_value = encode_object(None)
    response = create_message(GetObjectValueResponse,
            message, interface_name=interface_name, object_name=object_name,
            value=object_value)
    connection.send(response)

def process_activate_hack_command(message, sender, connection):
    hack = message["hack"]
    valid = True
    if hack == "buffer":
        print "Buffer hack activated for connection " + str(sender)
        connection.output_thread.enable_buffer_hack = True
    else:
        valid = False
    if valid:
        connection.send(create_message(ActivateHackResponse, message))
    else:
        connection.send_error(message, text="No such hack: " + str(hack))











from threading import Thread
import socket
import librtk
from Queue import Queue, Empty
import libautobus
import asyncore, asynchat
from traceback import print_exc
try:
    import json
except ImportError:
    import simplejson as json

"""
This module contains some protocols included with librtk.
"""

class ThreadedServer(Thread):
    def __init__(self, host, port, connect_function, **kwargs):
        """
        Creates a new ThreadedServer that will listen on the specified host
        and port and call the specified function when a new connection has
        been received. The argument to the function will be the connection
        itself.
        
        The specified keyword arguments will be passed into
        Connection.__init__. protocol and connect_function will already be
        specified, but values for any of the other arguments can be passed in.
        """
        Thread.__init__(self)
        self.host = host
        self.port = port
        self.connect_function = connect_function
        self.kwargs = kwargs
        self.socket = socket.socket()
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket.bind((host, port))
        self.socket.listen(5)
        self.running = True
    
    def run(self):
        while self.running:
            socket, _ = self.socket.accept()
            protocol = ThreadedProtocol(socket)
            connection = librtk.Connection(protocol, self.connect_function,
                    **self.kwargs)
            connection.start()
    
    def shutdown(self):
        self.running = False
        try:
            self.socket.close()
        except:
            pass

class ThreadedProtocol(object):
    def __init__(self, socket):
        self.socket = socket
    
    def protocol_init(self, connection):
        self.connection = connection
        self.out_queue = Queue()
        self.input_thread = libautobus.InputThread(self.socket,
                connection.protocol_receive, connection.protocol_connection_lost)
        self.output_thread = libautobus.OutputThread(self.socket,
                self.out_queue.get, shut_on_end=True)
        self.event_thread = librtk.EventThread()
    
    def protocol_start(self):
        self.input_thread.start()
        self.output_thread.start()
        self.event_thread.start()
    
    def protocol_send(self, data):
        self.out_queue.put(data)
    
    def protocol_event_ready(self, function):
        self.event_thread.schedule(function)
    
    def protocol_close(self):
        self.out_queue.put(None)
        self.event_thread.schedule(None)

# Async Implementation.

class AsyncDispatcher(asyncore.dispatcher):
    connection = None
    
    def __init__(self, host, port, connect_function, **kwargs):
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        
        assert host.__class__ == str
        assert port > 0
        assert port < 65536
        
        self.host = host
        self.port = port
        self.connect_function = connect_function
        self.kwargs = kwargs
        
        self.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.bind((host, port))
        
        self.listen(5)
    
    def handle_error(self):
    	raise
    
    def handle_accept(self):
        both = self.accept()
        if both is not None:
            sock, _ = both
            protocol = AsyncProtocol(sock)
            connection = librtk.Connection(protocol, self.connect_function)
            connection.start()
    
    def poll(self, timeout=0.0, map=None):
        if map is None:
            map = asyncore.socket_map
        
        assert map.__class__ is dict
        assert timeout.__class__ in (int, float)
        
        if hasattr(asyncore.select, "poll"):
            mypoll = asyncore.poll2
        else:
            mypoll = asyncore.poll
        
        mypoll(timeout, map)
        
        # Parse the event queue.
        
        for fds, obj in map.items():
            if obj.connection is None:
                continue
            try:
                while True: # We'll raise Empty when we have nothing else to run.
                    item = obj.async_eventq.get_nowait()
                    try:
                        item()
                    except:
                        print_exc()
            except Empty, AttributeError:
                continue
    
    def loop(self, timeout=10.0, use_poll=False, map=None):
        if map is None:
            map = asyncore.socket_map
        
        while True:
            if len(map) == 0:
                break
            self.poll(timeout=timeout, map=map)

class AsyncProtocol(asynchat.async_chat):
    def __init__(self, sock):
        assert sock.__class__ is socket.socket
        
        asynchat.async_chat.__init__(self, sock=sock)
        self.set_terminator("\n")
        self.recvq = ""
    
    def handle_error(self):
    	raise
    
    def collect_incoming_data(self, data):
        self.recvq += data
    
    def get_data(self):
        data, self.recvq = self.recvq, ""
        return data
    
    def found_terminator(self):
        data = self.get_data()
        self.connection.protocol_receive(json.loads(data))

    def protocol_init(self, connection):
    	self.connection = connection
        self.async_eventq = Queue()
    
    def protocol_start(self):
        pass
    
    def protocol_send(self, data):
        self.push(json.dumps(data))
    
    def protocol_event_ready(self, function):
        self.async_eventq.put_nowait(function)
    
    def protocol_close(self):
        self.close_when_done()

class LinkedProtocol(object):
    def __init__(self, link=None):
        """
        Creates one end of a linked connection. The first of the two ends
        should be created with link omitted, and the second one should be
        created with the the first one passed in as link. In other words, you
        could construct a pair of linked protocols like this:
        
        first = LinkedProtocol()
        second = LinkedProtocol(first)
        """
        self.other = None
        self.queue = Queue() # Holds messages going from other to self
        if link is not None:
            if link.other is not None:
                raise Exception("The specified linked protocol is already "
                        "linked to another protocol object.")
            self.other = link
            link.other = self
    
    def stream(self):
        for value in iter(self.queue.get, None):
            self.connection.protocol_receive(value)
        self.connection.protocol_connection_lost()
    
    def protocol_init(self, connection):
        self.connection = connection
        self.thread = Thread(target=self.stream)
        self.event_thread = librtk.EventThread()
    
    def protocol_start(self):
        self.thread.start()
        self.event_thread.start()
    
    def protocol_send(self, data):
        self.other.queue.put(data)
    
    def protocol_event_ready(self, function):
        self.event_thread.schedule(function)
    
    def protocol_close(self):
        self.event_thread.schedule(None)
        self.queue.put(None)
        self.other.queue.put(None)


















"""
A module used by Autolaunch to launch processes under different effective user ids.
"""
import os

def run(program, args, uid=None, gid=None, add_first_arg=True):
    args = args[:]
    if add_first_arg:
        args[0:0] = [program]
    if gid is not None:
        os.setgid(gid)
    if uid is not None:
        os.setuid(uid)
    os.execv(program, args)






"""
A program that shows a window titled "Hello" containing a label whose text
is "Hello, world! How are you?".
"""

def start(connection):
    w = connection.Window(title="Hello")
    w.close_request.listen(connection.close)
    l = connection.Label(w, text="Hello, world! How are you?")




import autobus
import special_objects

class AutobusInterface(object):
    """
    An interface built-in to the Autobus server that provides information about
    the server, such as the interfaces that are currently registered. To see a
    list of functions that are on this interface, call the function
    list_functions on this interface, passing in the string "autobus". The
    resulting list should provide plenty of information.
    """
    def list_interfaces(self):
        """
        Returns a list of all interfaces currently registered to the autobus
        server. The return value is a list of maps, with each map representing
        one interface. Each map has the following keys:
        
        owner: The numeric id of the connection that registered this interface.
        
        name: The name of the interface.
        
        special: True if this interface is special (I.E. registered by
        server-side code instead of by a client), false if it is a normal
        interface.
        
        doc: The documentation of the interface if the client supplied any such
        documentation, or the empty string if they didn't.
        """
        result = []
        for interface in autobus.interface_map.values():
            result.append({"owner": interface.connection.id if not
                    interface.special else 0, "name": interface.name,
                    "special": interface.special, "doc": interface.doc})
        return result
    
    def list_functions(self, interface_name):
        """
        Returns a list of all functions currently registered to the specified
        interface. If there's no such interface, an exception will be thrown.
        This function works correctly on special interfaces as well as normal
        interfaces.
        
        The return value is similar to the return value of list_interfaces().
        Each map within the list that's returned represents a function on the
        interface. 
        """
        try:
            interface = autobus.interface_map[interface_name]
        except KeyError:
            raise Exception("No such interface.")
        return interface.describe_functions()
    
    def list_events(self, interface_name):
        """
        Sane as list_functions, but returns a list of events available on this
        interface.
        """
        try:
            interface = autobus.interface_map[interface_name]
        except KeyError:
            raise Exception("No such interface.")
        return interface.describe_events()
        
    def list_objects(self, interface_name):
        """
        Same as list_functions, but returns a list of objects available on this
        interface. 
        """
        try:
            interface = autobus.interface_map[interface_name]
        except KeyError:
            raise Exception("No such interface.")
        return interface.describe_objects()
        
    def get_processed_message_count(self):
        """
        Returns the number of messages that Autobus has received and processed
        since it started up. This does not count messages that Autobus has
        sent; only inbound messages are counted.
        """
        return autobus.processed_message_count

interface = AutobusInterface()




from nextgen.keys.observing import Observable
from nextgen.keys import changes as cc
from afn.utils.singleton import Singleton as _Singleton
from afn.utils import slicer

_NONE = _Singleton("nextgen.keys.types._NONE")

class ObservableDict(Observable, dict):
    def _get_current_values(self, value_list):
        value_list += self.itervalues()
        super(ObservableDict, self)._get_current_values(value_list)
    
    def __delitem__(self, key):
        value = self[key]
        super(ObservableDict, self).__delitem__(key)
        self._notify_changed([cc.KeyRemoved(self, key, value)])
    
    def __setitem__(self, key, value):
        exists = key in self
        if exists:
            old = self[key]
        super(ObservableDict, self).__setitem__(key, value)
        if exists:
            self._notify_changed([cc.KeyUpdated(self, key, old, value)])
        else:
            self._notify_changed([cc.KeyAdded(self, key, value)])
    
    def clear(self):
        items = self.items()
        super(ObservableDict, self).clear()
        self._notify_changed([cc.KeyRemoved(self, k, v) for (k, v) in items])
    
    def copy(self):
        return ObservableDict(self.iteritems())
    
    def pop(self, key, default=_NONE):
        if key not in self:
            if default is _NONE:
                raise KeyError(key)
            else:
                return default
        value = self[key]
        del self[key] # This will call __delitem__, which will handle posting
        # the KeyRemoved event
        return value
    
    def popitem(self):
        if len(self) == 0:
            raise KeyError()
        key, value = self.iteritems().next()
        del self[key] # This will handle posting the KeyRemoved event
        return key, value
    
    def setdefault(self, key, default=None):
        if key not in self:
            self[key] = default # This will handle posting the KeyAdded event
            return default
        return self[key]
    
    def update(self, e, **f):
        changes = []
        if hasattr(e, "keys"):
            for key in e:
                value = e[key]
                if key in self:
                    changes.append(cc.KeyUpdated(self, key, self[key], value))
                else:
                    changes.append(cc.KeyAdded(self, key, value))
                super(ObservableDict, self).__setitem__(key, value)
        else:
            for key, value in e:
                if key in self:
                    changes.append(cc.KeyUpdated(self, key, self[key], value))
                else:
                    changes.append(cc.KeyAdded(self, key, value))
                super(ObservableDict, self).__setitem__(key, value)
        for key, value in f.iteritems():
            if key in self:
                changes.append(cc.KeyUpdated(self, key, self[key], value))
            else:
                changes.append(cc.KeyAdded(self, key, value))
            super(ObservableDict, self).__setitem__(key, value)
        self._notify_changed(changes)
    
    def __str__(self):
        return self.__repr__()
    
    def __repr__(self):
        return "<ObservableDict: %s>" % super(ObservableDict, self).__repr__()


class ObservableList(Observable, list):
    def _get_current_values(self, value_list):
        value_list += self
        super(ObservableList, self)._get_current_values(value_list)
    
    # TODO: finish this up, and use afn.utils.slicer to implement slicing, and
    # see if Py3 has that and submit it as a patch if it doesn't
    
    def __delitem__(self, s):
        if isinstance(s, (int, long)):
            s = slice(s, s + 1)
        indexes = list(slicer(len(self), s.start, s.stop, s.step))
        indexes.sort()
        changes = []
        # We need to increasingly subtract from each index in the change in
        # order to reflect that the items before it have already been deleted
        # and therefore the items after them have been shifted down one position
        for e_index, i in enumerate(indexes):
            changes.append(cc.ItemRemoved(self, i - e_index, self[i]))
        super(ObservableList, self).__delitem__(s)
        self._notify_changed(changes)
    
    def __delslice__(self, start, stop):
        self.__delitem__(slice(start, stop))
    
    def __iadd__(self, items):
        index = len(self)
        super(ObservableList, self).__iadd__(items)
        changes = [cc.ItemInserted(self, i, self[i]) for i in range(index, len(self))]
        self._notify_changed(changes)
        return self
    
    def __imul__(self, number):
        index = len(self)
        super(ObservableList, self).__imul__(number)
        changes = [cc.ItemInserted(self, i, self[i]) for i in range(index, len(self))]
        self._notify_changed(changes)
        return self
    
    def __setitem__(self, s, item):
        if isinstance(s, (int, long)):
            old = self[s]
            super(ObservableList, self).__setitem__(s, item)
            self._notify_changed([cc.ItemUpdated(self, s, old, item)])
        else:
            raise NotImplementedError("Assignments to list slices of "
                                      "ObservableList instances haven't been "
                                      "implemented yet.")
    
    def __setslice__(self, start, stop, item):
        self.__setitem__(slice(start, stop), item)
    
    def append(self, item):
        super(ObservableList, self).append(item)
        self._notify_changed([cc.ItemInserted(self, len(self) - 1, item)])
    
    def extend(self, iterable):
        # As far as I can tell, extend and __iadd__ behave the exact same way
        self += iterable
    
    def insert(self, index, item):
        if index > len(self):
            index = len(self)
        while index < 0:
            index += len(self) or 1 # Increment by 1 if this list is empty
        super(ObservableList, self).insert(index, item)
        self._notify_changed([cc.ItemInserted(self, index, item)])
    
    def pop(self, index=None):
        if index is None:
            index = len(self) - 1
        item = self.pop(index)
        self._notify_changed([cc.ItemRemoved(self, index, item)])
    
    def remove(self, value):
        del self[self.index(value)] # __del__ will handle posting the
        # ItemRemoved notification for us
    
    def reverse(self):
        super(ObservableList, self).reverse()
        length = len(self)
        highest = length - 1
        changes = [cc.ItemsSwapped(self, highest - i, i, self[i], self[highest - i]) for i in range(length / 2)]
        self._notify_changed(changes)
    
    def relocate(self, source, target):
        """
        Moves the item at the specified source index to the specified target
        index. This is essentially equivalent to:
        
        item = self[source]
        del self[source]
        self.insert(target, item)
        
        but it causes an ItemRelocated notification to be posted instead of an
        ItemRemoved followed by an ItemAdded.
        """
        if source > len(self) - 1 or source < 0:
            raise ValueError("Out of bounds relocation source: " + str(source))
        item = self[source]
        if target > len(self) - 1 or target < 0:
            raise ValueError("Out of bounds relocation target: " + str(target))
        super(ObservableList, self).__delitem__(source)
        super(ObservableList, self).insert(target, item)
        self._notify_changed([cc.ItemRelocated(self, source, target, item)])
    
    def sort(self, *args):
        raise NotImplementedError("Sorting an OrderedList is not currently "
                                  "supported. It will be supported at some "
                                  "point in the near future.")
    
    def __str__(self):
        return self.__repr__()
    
    def __repr__(self):
        return "<ObservableList: %s>" % super(ObservableList, self).__repr__()













































import sys
from traceback import print_exc
# sys.path.append("/afn/afn-python/src")

from parallel import Parallel #@UnresolvedImport
from autobus2 import Bus
from time import sleep

def main():
    p = Parallel()
    previous = True
    with Bus() as bus:
        with bus.get_service_proxy({"type": "speak"}, multiple=True) as s: 
            while True:
                current = p.getInPaperOut()
                if current != previous:
                    previous = current
                    if not current: # Button was just pressed (i.e. the paperOut pin
                        # was just shorted to ground)
                        print "Doorbell was pressed"
                        try:
                            print s["say_text"]("someone is ringing the_front doorbell", callback=None)
                        except:
                            print_exc()
                        sleep(5)
                sleep(0.03)







from libautobus import AutobusConnection
from os import getpid

class ExampleInterface(object):
    def say_hello(self, name):
        return "Hello, " + name + ". How are you?"

for i in range(10):
    server = AutobusConnection()
    server.add_interface("afntest.autobus3." + str(i), ExampleInterface())
    server.connect()

print "Started up. pid is " + str(getpid()) + " which you can use to kill"
print "this once you're done with it."



import gtk
import gtk.gdk
import librtkclient
import gobject
from urlparse import urlparse
import sys
from librtk.constants import DEFAULT_PORT
from socket import socket as Socket, error as SocketError
from librtk.protocols import ThreadedProtocol
from Queue import Queue, Empty
from traceback import print_exc

# gobject.timeout_add(interval, function, ...)

gobject.threads_init()

class Widget(librtkclient.Widget):
    def destroy(self):
        self.widget.destroy()


class Window(Widget):
    def setup(self):
        self.widget = gtk.Window()
        if "title" in self.widget_properties:
            self.widget.set_title(self.widget_properties["title"])
        self.widget.set_default_size(1, 1)
        self.widget.connect("delete-event", self.window_close)
        self.widget.show()
    
    def post_setup(self, child):
        self.widget.add(child.widget)
    
    def window_close(self, *args):
        self.send_event("close_request", True)
        return True


class VBox(Widget):
    def setup(self):
        self.widget = gtk.VBox(spacing=1)
        self.widget.show()
    
    def post_setup(self, child):
        self.widget.add(child.widget)


class Label(Widget):
    def setup(self):
        self.widget = gtk.Label(self.widget_properties.get("text", ""))
        self.widget.show()
    
    def update_widget(self, properties):
        if "text" in properties:
            self.widget.set_text(properties["text"])


class Button(Widget):
    def setup(self):
        self.widget = gtk.Button(self.widget_properties.get("text", ""))
        self.widget.connect("clicked", self.clicked)
        self.widget.show()
    
    def update_widget(self, properties):
        if "text" in properties:
            self.widget.set_label(properties["text"])
    
    def clicked(self, *args):
        self.send_event("clicked", True)


class TextBox(Widget):
    def setup(self):
        self.widget = gtk.Entry()
        self.widget.set_text(self.widget_properties.get("text", ""))
        self.widget.set_width_chars(self.widget_properties.get("width", 20))
        self.widget.connect("changed", self.text_changed)
        self.widget.connect("activate", self.enter_pressed)
        self.widget.show()
    
    def update_widget(self, properties):
        if "width" in properties:
            self.widget.set_width_chars(properties["width"])
    
    def text_changed(self, *args):
        self.send_set_state(text=self.widget.get_text())
    
    def call(self, name, args):
        if name == "set_text":
            text, = args
            self.widget.set_text(text)
        else:
            self.connection.fatal_error("Invalid call " + name + " on TextBox")
    
    def enter_pressed(self, *args):
        pass


widget_list = [Window, Label, VBox, Button, TextBox]
widget_set = dict([(w.__name__, w) for w in widget_list])
feature_set = ["widget:" + w for w in widget_set.keys()]


def main():
    if len(sys.argv) <= 1:
        print "You need to specify the url of the application to display."
        return
    scheme, location = urlparse(sys.argv[1])[0:2]
    if scheme != "rtk":
        print ("Only rtk:// urls are supported for now. I might add an HTTP "
                "transport at some point in the future.")
    if ":" not in location:
        location += ":" + str(DEFAULT_PORT)
    host, port = location.split(":")
    port = int(port)
    socket = Socket()
    try:
        socket.connect((host, port))
    except SocketError, e:
        print "Error while connecting: " + str(e)
        return
    protocol = ThreadedProtocol(socket)
    event_queue = Queue()
    def close():
        gtk.main_quit()
    connection = librtkclient.Connection(protocol, event_queue.put, close,
            feature_set, widget_set)
    connection.start()
    def idle():
        try:
            while True:
                event_queue.get(block=False)()
        except Empty:
            pass
        except:
            print_exc()
        return True
    gtk.timeout_add(100, idle)
    try:
        gtk.main()
    except KeyboardInterrupt:
        print "Interrupted, shutting down"
    connection.close()
    print "Terminated."
    

































from collections import namedtuple

KeyAdded = namedtuple("KeyAdded", ["key", "new"])
KeyUpdated = namedtuple("KeyUpdated", ["key", "old", "new"])
KeyRemoved = namedtuple("KeyRemoved", ["key", "old"])

ItemInserted = namedtuple("ItemInserted", ["index", "new"])
ItemUpdated = namedtuple("ItemUpdated", ["index", "old", "new"])
ItemRemoved = namedtuple("ItemRemoved", ["index", "old"])
ItemRelocated = namedtuple("ItemRelocated", ["source", "target", "value"])
ItemsSwapped = namedtuple("ItemsSwapped", ["i", "j", "first", "second"])



"""
A program that shows a label with a number and a button. Every time the
button is clicked, the number on the label increments. Multiple clients
do not share the same number count; see shared_count for an example where
multiple clients do.
"""

def start(connection):
    value = [0] # Store as a list so that it's mutable from closures
    window = connection.Window(title="The Counter")
    window.close_request.listen(connection.close)
    box = connection.VBox(window)
    label = connection.Label(box, text=str(value[0]))
    button = connection.Button(box, text="Click to increment the counter.")
    def clicked():
        value[0] += 1
        label.text = str(value[0])
    button.clicked.listen(clicked)




# FIXME: change everything that points to the proxy module to point to the
# single and multiple modules instead, then get rid of these two wildcard
# import lines

from autobus2.proxy.single import *
from autobus2.proxy.multiple import *





broadcast_port = 52722
broadcast_interval = 10
# Between 0 and broadcast_random seconds will be randomly added to
# broadcast_interval between each broadcast. Making sure that broadcasts happen
# at random times reduces the chance of broadcasts colliding on a network
# filled with Autobus services.
broadcast_random = 5
broadcast_receiver_timeout = 1
query_initial_intervals = [0.1, 0.5, 1, 2, 4, 7]
query_response_random = 0.2 # Same as broadcast_random, but specifies up to how
# many seconds may be delayed between receiving a query and sending a response
# to that query.


#!/usr/bin/env python

import sys
from os.path import join, dirname, realpath
afn_path = dirname(realpath(__file__))
sys.path.append(join(afn_path, "src"))
if len(sys.argv) <= 1:
    print 'usage: run python_module_name'
    print 'adds "src" to the path, then imports the specified module and calls'
    print 'its main function with no arguments. The argument representing the'
    print 'module name will be removed from sys.argv first so that additional'
    print 'arguments can be provided to the module.'
    sys.exit(0)
module_name = sys.argv[1]
del sys.argv[1]
module = __import__(module_name)
if "." in module_name:
    subnames = module_name.split(".")[1:]
    for subname in subnames:
        module = getattr(module, subname)
module.main()




class Partial(object):
    """
    A replacement for functools.partial that implements __eq__ and __ne__; a
    Partial object is equal to another Partial if their underlying functions
    are equal, both contain the same number of arguments and keyword arguments,
    and their arguments and keyword arguments compare equal.
    """
    def __init__(*args, **kwargs): #@NoSelf
        self = args[0]
        self.function = args[1]
        self.args = args[2:]
        self.kwargs = kwargs
    
    def __call__(*args, **kwargs): #@NoSelf
        self = args[0]
        args = args[1:]
        kwtemp = self.kwargs.copy()
        kwtemp.update(kwargs)
        return self.function(*(self.args + args), **kwtemp)
    
    def __eq__(self, other):
        if not isinstance(other, Partial):
            return NotImplemented
        return (self.function == other.function and 
                self.args == other.args and 
                self.kwargs == other.kwargs)
    
    def __ne__(self, other):
        if not isinstance(other, Partial):
            return NotImplemented
        return not self.__eq__(other)


def partial(*args, **kwargs):
    """
    A function that simply returns Partial(*args, **kwargs). This allows this
    module to function as a drop-in replacement for functools insofar as using
    partial goes.
    """
    return Partial(*args, **kwargs)









from autobus2 import Bus
import time

def main():
    with Bus() as bus:
        service = bus.create_service({"autobus.example": "testobjectserver2"})
        object = service.create_object("test", time.ctime(time.time()))
        service.activate()
        try:
            while True:
                time.sleep(5)
                object.set_value(time.ctime(time.time()))
        except KeyboardInterrupt:
            pass



from autobus2 import Bus
import multiprocessing as mp

class Service(object):
    pass

def main():
    with Bus() as bus:
        bus.create_service({"type": "autolaunch"}, from_py_object=Service())
        



from threading import RLock, Condition
import autobus2
from autobus2 import exceptions, common
from afn.utils import wrap, print_exceptions
from afn.utils.partial import partial
from afn.utils.concurrent import synchronized_on
from afn.utils.multimap import Multimap as _Multimap
from Queue import Queue

class MultipleServiceProxy(common.AutoClose):
    def __init__(self, bus, info_filter, bind_function=None, unbind_function=None):
        self.bus = bus
        self.info_filter = info_filter
        self.lock = RLock()
        self.bind_condition = Condition(self.lock)
        self.bind_function = bind_function
        self.unbind_function = unbind_function
        self.service_map = {} # Map of service ids to connections
        self.object_watchers = _Multimap()
        self.is_alive = True
        bus.add_service_listener(self.service_event, info_filter, True)
    
    @synchronized_on("lock")
    def service_event(self, service_id, host, port, info, event):
        if not self.is_alive:
            return
        getattr(self, "service_" + event.short_name.lower())(service_id, host, port, info)
    
    def service_discovered(self, service_id, host, port, info):
        self.connect_to(service_id, host, port, info)
    
    def service_changed(self, service_id, host, port, info):
        self.disconnect_from(service_id)
        self.connect_to(service_id, host, port, info)
    
    def service_undiscovered(self, service_id, host, port, info):
        self.disconnect_from(service_id)
    
    def connect_to(self, service_id, host, port, info):
        connection = self.bus.connect(host, port, service_id,
                open_listener=partial(self.connection_opened, info),
                close_listener=partial(self.connection_closed, info),
                lock=self.lock)
        for name, watchers in self.object_watchers.items():
            for watcher in watchers:
                connection.watch_object(name, partial(watcher, connection, info))
        self.service_map[service_id] = connection
    
    def disconnect_from(self, service_id):
        self.service_map[service_id].close()
        del self.service_map[service_id]
    
    def connection_opened(self, info, connection):
        with print_exceptions:
            if self.bind_function:
                self.bind_function(self, connection, info)
    
    def connection_closed(self, info, connection):
        with print_exceptions:
            if self.unbind_function:
                self.unbind_function(self, connection, info)
    
    @synchronized_on("lock")
    def close(self):
        if not self.is_alive: # Already closed
            return
        self.is_alive = False
        for service_id in list(self.service_map.keys()):
            self.disconnect_from(service_id)
        self.bus.remove_service_listener(self.service_event)
    
    def __getitem__(self, name):
        return MultipleServiceFunction(self, name)
    
    @property
    def live_connections(self):
        with self.lock:
            for connection in list(self.service_map.values()):
                if connection.is_connected:
                    yield connection
    
    @synchronized_on("lock")
    def watch_object(self, name, function):
        """
        function -> (connection, info, value) -> None
        """
        self.object_watchers.add(name, function)
        for info, connection in self.service_map.items():
            connection.watch_object(name, partial(function, connection, info))
    
    @synchronized_on("lock")
    def unwatch_object(self, name, function):
        """
        """
        self.object_watchers.remove(name, function)
        for info, connection in self.service_map.items():
            connection.unwatch_object(name, partial(function, connection, info))


class MultipleServiceFunction(object):
    def __init__(self, proxy, name):
        self.proxy = proxy
        self.name = name
    
    def __call__(self, *args, **kwargs):
        with self.proxy.lock:
            callback = kwargs.get("callback", autobus2.SYNC)
            timeout = kwargs.get("timeout", 30)
            safe = kwargs.get("safe", False)
            if callback is None:
                count = 0
                for connection in self.proxy.live_connections:
                    count += 1
                    connection[self.name](*args, callback=None, safe=safe)
                return count
            elif callback is not autobus2.SYNC:
                count = 0
                for connection in self.proxy.live_connections:
                    count += 1
                    connection[self.name](*args, callback=partial(callback, connection.service_id), safe=safe)
                return count
            # SYNC mode.
            # This is a bit more complicated. What we're going to do is
            # create one queue for each connection, then call the functions
            # on the connections with each queue's put method as the
            # callback.
            queues = dict((c.service_id, Queue()) for c in self.proxy.live_connections)
            for c in self.proxy.live_connections:
                c[self.name](*args, callback=queues[c.service_id].put, safe=True)
        # We drop out of self.proxy.lock so that we can block on the queues
        results = {}
        for service_id in queues:
            # FIXME: This causes the timeout to be applied once for each
            # service that we're actually contacting. What we need to do is
            # keep track of when we start blocking and then only block how much
            # time is left until the timeout would have expired.
            try:
                results[service_id] = queues[service_id].get(timeout=timeout)
            except Exception as e:
                results[service_id] = e
        # Scan through and accumulate any exceptions into a map
        exception_map = {}
        for service_id, value in results.items():
            if isinstance(value, Exception):
                exception_map[service_id] = value
        # Then, if there were any exceptions to accumulate, throw an exception
        # containing them all
        if exception_map:
            raise exceptions.CommandErrorException(exception_map)
        # No exception happened, so return the results
        return results





from libautobus import AutobusConnection
from autoconfigure import Configuration

class Interface(object):
    """
    The home daemon is a higher-level interface to ActiveHome. It allows
    modules to be configured, turned on, and turned off. It supports various
    types of modules, such as cameras (turning on any camera will cause all
    other cameras to be turned off) and chimes (turning off a chime has no
    effect, but it can be turned on multiple times).
    
    Homed delegates all of its control functions to activehomed. An instance
    of activehomed must be running on the same Autobus server that homed is
    attached to.
    """
    def on(self, address):
        """
        Turns the specified module on. This module must be in the configuration
        file. This function will take special module types into effect; for
        example, calling this function on a camera will automatically turn all
        other cameras off.
        """
        address = address.lower()
        verify_existing_address(address)
        # Assume powerline for now. In the future, we'll want to have some
        # configuration variable that specifies whether this is powerline
        # or rf or what.
        activehome.action.send("sendplc", address, "on")
    
    def off(self, address):
        """
        Turns the specified module off. This follows the same general contract
        as the on function.
        """
        address = address.lower()
        verify_existing_address(address)
        activehome.action.send("sendplc", address, "off")

def verify_existing_address(address):
    if len(address) != 2:
        raise Exception("Addresses have to be exactly 2 characters in length.")
    if not config.has_section(address):
        raise Exception("The specified address is not present in the "
                "configuration. If you want to send X10 commands irrespective "
                "of whether they're in configuration, check out the action "
                "function of the activehome interface.")

def main():
    global bus
    global config
    global activehome
    bus = AutobusConnection()
    activehome = bus["activehome"]
    bus.add_interface("home", Interface())
    config = Configuration(bus, "configure.home", "homed.conf")
    bus.start_connecting()
    bus.interrupt_loop()
    



"""
This is a pseudo-module to hold all widget classes generated by librtk for
any given connection. The classes aren't really present in this module; it
simply exists as a placeholder for the module that these generated classes
appear to be defined in.
"""




"""
A program that shows five labels stacked one on top of the other.
"""

def start(connection):
    w = connection.Window(title="Hello")
    w.close_request.listen(connection.close)
    box = connection.VBox(w)
    for i in range(1,6):
        connection.Label(box, text="Hello number " + str(i))





"""
A module for parsing schemas in XML format. The default widget schema uses
this.
"""

from xml.etree import ElementTree

"""
A module for parsing XML documents specifying widget schemas into the
corresponding schema objects used by librt itself.

WARNING: Parsing a foreign schema is unsafe. Parts of the schems are passed
into Python's eval function, so a schema document could execute arbitrary
code. Do not parse untrusted schemas.
"""

import structure
import categories

def parse_file(filename):
    with open(filename) as file:
        return parse(file.read())

def parse(xml):
    root = ElementTree.fromstring(xml)
    schema = {}
    for element in root:
        widget = parse_widget(element)
        schema[widget.name] = widget
    return schema

def parse_widget(element):
    name = element.attrib["name"]
    doc = get_doc(element)
    type = {"toplevel":categories.TOPLEVEL, "container":categories.CONTAINER,
            "widget":categories.WIDGET}[element.tag]
    widget_props = {}
    layout_props = {}
    state_props = {}
    calls = {}
    events = {}
    for e in element.findall("widget"):
        widget_props[e.attrib["name"]] = structure.WidgetPropertySchema(
                name=e.attrib["name"],
                doc=get_doc(e),
                writable=e.attrib.get("writable", "true") == "true",
                default=eval(e.attrib.get("default", None)))
    for e in element.findall("layout"):
        layout_props[e.attrib["name"]] = structure.LayoutPropertySchema(
                name=e.attrib["name"],
                doc=get_doc(e),
                writable=e.attrib.get("writable", "true") == "true",
                default=eval(e.attrib.get("default", None)))
    for e in element.findall("state"):
        state_props[e.attrib["name"]] = structure.StatePropertySchema(
                name=e.attrib["name"],
                doc=get_doc(e),
                default=eval(e.attrib.get("default", '""')))
    for e in element.findall("call"):
        calls[e.attrib["name"]] = structure.CallSchema(
                name=e.attrib["name"],
                doc=get_doc(e))
    for e in element.findall("event"):
        events[e.attrib["name"]] = structure.EventSchema(
                name=e.attrib["name"],
                doc=get_doc(e))
    return structure.WidgetSchema(name=name, doc=doc, type=type,
            widget_properties=widget_props, layout_properties=layout_props,
            state_properties=state_props, calls=calls, events=events)

def get_doc(element):
    if "doc" in element.attrib:
        return element.attrib["doc"]
    doc_child = element.find("doc")
    if doc_child is not None:
        return doc_child.text
    return ""
























from objectivejson import Serializer
serial = Serializer()

@serial.serializable("example")
class Example(object):
    serial_info = [
        ["first", int, True],
        ["second", (int, str), False]
    ]

e = Example()
e.first = 5
print serial.serialize(e)
e.second = "hello"
print serial.serialize(e)
e.second = 8
print serial.serialize(e)
del e.first
print serial.serialize(e)
e.first = None
print serial.serialize(e)
# This one should raise an exception
e.second = None
print serial.serialize(e)
print "It didn't throw an exception, which it should have"




import sys
from traceback import print_exc
from datetime import datetime # For use with :something arguments
from optparse import OptionParser
from time import sleep

description = """\
A command-line Autobus 1 client. Autosend allows a variety of actions to be
performed against an Autobus server.

(If you want an Autobus 2 command-line client, use autosend2.)

One, and only one, mode option can be specified. This tells autosend what to
do.

Some modes use an interface name. Some use an interface and and item. Some can
also take arguments. Each command-line parameter after the item name is
interpreted as an argument. Arguments are interpreted as follows: if the
argument can be parsed as an integer, it will be sent as an integer. If the
argument is equal to "true", "false", or "null", it will be sent as a boolean
for the former two and as null for the latter. If the argument starts with
a : character, the rest of the argument will be interpreted as a Python
expression whose value will be used as the argument's value. Otherwise, the
argument is interpreted as a string.
"""

parser = OptionParser(usage=
        "usage: autosend [-?] [options] [interface [item [arguments...]]]",
        description="!!!INFO!!!",
        add_help_option=False)
parser.add_option("-h", type="string", action="store", dest="host", help="The host to connect "
        "to. The default is whatever you've configured in libautobus.conf, or "
        "localhost if you haven't created libautobus.conf.", default=None)
from libautobus import DEFAULT_PORT
parser.add_option("-p", type="int", dest="port", help="The port to connect to. "
        "The default is whatever you've configured in libautobus.conf, or "
        + str(DEFAULT_PORT) +
        " if you haven't created libautobus.conf.", default=None)
parser.add_option("-l", action="store_const", const="list", dest="mode",
        help="List mode. No additional command-line arguments are needed. This "
        "is the default mode if no command-line options are specified. In "
        "this mode, Autosend will print out a list of all interfaces currently "
        "registered to the Autobus server, along with their documentation.")
parser.add_option("-i", action="store_const", const="interface", dest="mode",
        help="Interface information mode. Only an interface needs to be "
        "specified; if an item is specified, it will be ignored. This is the "
        "default mode if only an interface is specified. In this mode, "
        "the list of functions, events, and objects present on the specified "
        "remote interface will be printed to stdout along with the documentation "
        "for those items.")
parser.add_option("-f", action="store_const", const="function", dest="mode",
        default="unspecified", help="Function mode. This is the default if an "
        "interface and an item are both specified. This causes the function "
        "specified as the item to be called with the specified arguments. The "
        "function's return value will be printed out.")
parser.add_option("-e", action="store_const", const="event", dest="mode",
        help="Event mode. This attaches to the event specified as the item on "
        "the specified interface. Whenever the event is fired, the event's "
        "arguments will be printed to stdout, followed by a blank line.")
parser.add_option("-o", action="store_const", const="object", dest="mode",
        help="Object mode. This gets the current value of the specified "
        "object and prints it to stdout, then exits.")
parser.add_option("-w", action="store_const", const="watch", dest="mode",
        help="Object watch mode. This gets the current value of the specified "
        "object and prints it to stdout, then waits. Every time the object's "
        "value changes, the new value will be printed to stdout. Like -e, "
        "you'll have to use Ctrl+C to kill autosend when you're done watching "
        "the object's value change.")
parser.add_option("-?", "--help", action="store_const", const="help", dest="mode")
parser.disable_interspersed_args()
options, command_line_args = parser.parse_args()

mode = options.mode
if mode == "unspecified":
    if len(command_line_args) == 0:
        mode = "list"
        print "Use autosend --help for more information.\n"
    elif len(command_line_args) == 1:
        mode = "interface"
    elif len(command_line_args) >= 2:
        mode = "function"

if mode == "help":
    print parser.format_help(None).replace("!!!INFO!!!", description)
    sys.exit()

connect_options = {}
if options.host is not None:
    connect_options["host"] = options.host
if options.port is not None:
    connect_options["port"] = options.port

fixed_values = {"true": True, "false": False, "null": None}
arguments = []

interface_name = None
item_name = None
if len(command_line_args) == 0:
    pass
elif len(command_line_args) == 1:
    interface_name = command_line_args[0]
    command_line_args = []
elif len(command_line_args) >= 2:
    interface_name = command_line_args[0]
    item_name = command_line_args[1]
    command_line_args = command_line_args[2:]

for value in command_line_args:
    # Try parsing as an int
    try:
        arguments.append(int(value))
        continue
    except: # Not an int
        pass
    if value in fixed_values:
        arguments.append(fixed_values[value])
        continue
    if value.startswith(":"):
        arguments.append(eval(value[1:]))
        continue
    arguments.append(value)    
    

from libautobus import AutobusConnection, NotConnectedException
bus = AutobusConnection(**connect_options)

# We've got everything set up thus far. Now we'll look up the mode and figure
# out what we need to do before we connect to the Autobus server.

if mode == "event":
    def event_function(*arguments):
        if len(arguments) < 0:
            print "Function fired with no arguments."
        print str(arguments)
    bus.add_event_listener(interface_name, item_name, event_function)

if mode == "object":
    printed = False
    def object_function(new_value):
        global printed
        if printed:
            return
        printed = True
        print str(type(new_value))
        print str(new_value)
        bus.shutdown()
    bus.add_object_watch(interface_name, item_name, object_function)
elif mode == "watch":
    def object_function(new_value):
        print str(type(new_value))
        print str(new_value)
    bus.add_object_watch(interface_name, item_name, object_function)

try:
    bus.connect()
except NotConnectedException:
    print "Couldn't connect to the Autobus server."
    sys.exit()


if mode == "list":
    try:
        interfaces = bus["autobus"].list_interfaces()
    except Exception as e:
        print "Exception while getting interface list: " + str(e)
        if e.__class__ is not Exception:
            print_exc()
        bus.shutdown()
        sys.exit()
    interfaces.sort(cmp=lambda x, y: cmp(x["name"], y["name"]))
    if len(interfaces) == 0:
        print "No interfaces currently available."
    for interface in interfaces:
        print "Interface " + interface["name"] + ":"
        print interface["doc"]
        print "-" * 79
    bus.shutdown()
elif mode == "interface":
    #TODO: list objects and events as well
    try:
        functions = bus["autobus"].list_functions(interface_name)
        events = bus["autobus"].list_events(interface_name)
        objects = bus["autobus"].list_objects(interface_name)
    except Exception as e:
        print "Exception while getting interface member list: " + str(e)
        if e.__class__ is not Exception:
            print_exc()
        bus.shutdown()
        sys.exit()
    functions.sort(cmp=lambda x, y: cmp(x["name"], y["name"]))
    events.sort(cmp=lambda x, y: cmp(x["name"], y["name"]))
    objects.sort(cmp=lambda x, y: cmp(x["name"], y["name"]))
    def print_object_info(type, items):
        if len(items) == 0:
            print "No %ss currently available on that interface." % type
        for index, item in enumerate(items):
            print "%s%s %s:" % (type[0:1].upper(), type[1:], item["name"])
            print item["doc"]
            if index != (len(items) - 1):
                print "-" * 79
    print_object_info("function", functions)
    print "=" * 79
    print_object_info("event", events)
    print "=" * 79
    print_object_info("object", objects)
    bus.shutdown()
elif mode == "function":
    interface = bus[interface_name]
    function = getattr(interface, item_name)
    try:
        return_value = function(*arguments)
        if return_value is None:
            print "Function ran successfully with no return value."
        else:
            print str(type(return_value))
            print str(return_value)
    except Exception as e:
        if e.__class__ is Exception:
            print "Exception in remote function: " + str(e)
        else:
            print "Local exception:"
            print_exc()
    bus.shutdown()
elif mode == "object" or mode == "watch" or mode == "event":
    # The listeners added before we added will do everything for us, so we just
    # have to wait and watch for keyboard interrupts
    try:
        while not bus.is_shut_down: # Object mode shuts down the bus after
            # it's done
            sleep(0.25)
    except KeyboardInterrupt:
        bus.shutdown()







import autobus2
from time import sleep

def changed(*args):
    print "CHANGED: " + str(args)

def main():
    with autobus2.Bus() as bus:
        sleep(1)
        with bus.connect_to({"autobus.example": "testobjectserver1"}) as connection:
            connection.wait_for_connect()
            connection.watch_object("test", changed)
            autobus2.wait_for_interrupt()




from threading import RLock, Condition
import autobus2
from autobus2 import exceptions, common
from afn.utils import wrap, print_exceptions
from afn.utils.concurrent import synchronized_on
from functools import partial

class SingleServiceProxy(common.AutoClose):
    def __init__(self, bus, info_filter):
        self.bus = bus
        self.info_filter = info_filter
        self.lock = RLock()
        self.bind_condition = Condition(self.lock)
        self.service_map = {}
        self.connection = None
        self.is_alive = True
        self.service_event_wrapper = wrap(self.service_event)
        bus.add_service_listener(self.service_event_wrapper, info_filter, True)
    
    @synchronized_on("lock")
    def service_event(self, service_id, host, port, info, event):
        if not self.is_alive:
            return
        getattr(self, "service_" + event.short_name.lower())(service_id, host, port, info)
    
    def service_discovered(self, service_id, host, port, info):
        self.service_map[service_id] = (host, port)
        if self.connection is None:
            self.connect_to(service_id)
    
    def service_changed(self, service_id, host, port, info):
        self.service_map[service_id] = (host, port)
        if self.connection is not None and self.connection.service_id == service_id:
            self.connect_to(service_id)
    
    def service_undiscovered(self, service_id, host, port, info):
        del self.service_map[service_id]
        if self.connection is not None and self.connection.service_id == service_id:
            self.connect_to(None)
    
    def connect_to(self, service_id):
        if self.connection is not None:
            self.connection.close()
            self.connection = None
        if service_id:
            host, port = self.service_map[service_id]
            self.connection = self.bus.connect(host, port, service_id, open_listener=self.connection_opened,
                                               close_listener=self.connection_closed, lock=self.lock)
    
    def connection_opened(self, connection):
        with self.lock:
            self.bind_condition.notify_all()
    
    def connection_closed(self, connection):
        pass
    
    def __getitem__(self, name):
        return SingleServiceFunction(self, name)
    
    @synchronized_on("lock")
    def close(self):
        if not self.is_alive: # Already closed
            return
        self.is_alive = False
        self.connect_to(None)
        self.bus.remove_service_listener(self.service_event_wrapper)
    
    def wait_for_bind(self, timeout=10):
        with self.lock:
            if self.connection:
                if self.connection.is_connected:
                    return
            self.bind_condition.wait(timeout)
            if self.connection:
                if self.connection.is_connected:
                    return
            raise exceptions.TimeoutException()
    
    @property
    @synchronized_on("lock")
    def current_service_id(self):
        if self.connection:
            return self.connection.service_id
        return None


class SingleServiceFunction(object):
    def __init__(self, proxy, name):
        self.proxy = proxy
        self.name = name
    
    def __call__(self, *args, **kwargs):
        with self.proxy.lock:
            if self.proxy.connection is None:
                raise exceptions.NotConnectedException
            function = self.proxy.connection[self.name]
        return function(*args, **kwargs)





import functools

def tracker(track_list):
    """
    A decorator that can be used on an RTK connect function thus:
    
    connections = []
    @tracker(connections)
    def start(connection):
        ...
    
    Every connection passed into the method will be added to the specified
    list. A listener is registered on every such connection that will cause
    it to be removed from the list when it closes.
    """
    def decorator(function):
        def wrapper(connection):
            def close():
                track_list.remove(connection)
            connection.add_close_function(close)
            track_list.append(connection)
            function(connection)
        functools.update_wrapper(wrapper, function)
        return wrapper
    return decorator



















"""
This module is a main module. It provides a Protocol Buffers compiler for Java.
The resulting Java code is much simpler and easier to use than the Java code
that Google's protoc generates. Message classes are mutable, which makes things
a ton easier to use, but they also have setters that return the message, which
allows for chaining (new SomeMessageType().setX(...).setY(...).setZ(...)).

Right now, this only supports really basic protobuf functionality. It does not
check a good majority of stuff that protoc generally should, instead relying on
the Java compiler to do this. It does not require any external libraries to be
on the classpath; the generated code contains everything needed to perform
encoding. (It heavily relies on DataInputStream and DataOutputStream for
encoding and decoding functions.)

Multiple messages in a single file are supported (and they can reference each
other), but nested messages (messages defined within messages) are not. Enums
are not yet supported. Required, optional, and repeated fields are all fully
supported. Defaults are not; fields in the generated classes are stored as
their Java wrapper types, and if they're not present in the message, they'll be
null (unless they're marked as required, in which case the class will throw an
exception when trying to deserialize the message). Unknown fields are not
allowed; if an unknown field is encountered while deserializing, an exception
will be thrown. If someone wants to add support for unknown fields, I'd greatly
appreciate it.

Optional and required fields' classes are their wrapper type or the generated
class of the message for a compound field. Repeated fields are lists of the
corresponding types for that entry. These fields are initialized to contain
ArrayList instances, but any list instance is fine for the serializer; it's
perfectly fine for some user of the class to replace the list with a
LinkedList, for example, if they see fit.

The java_package and java_outer_classname options must be specified in the
protobuf file, and they must be specified before any other messages. All
options, for that matter, must be specified before messages are defined. This
is a limitation I might remove at some point.

The supported primitive types are string, fixed32, fixed64, double, and bool.

Only single-line comments (I.E. those starting with "//") are supported.

Field numbers must be less than 2047.

This module uses pyparsing to parse the protobuf file. You'll need pyparsing
installed in order for this module to work. 
"""

from pyparsing import Literal, Regex, ZeroOrMore, Forward, Optional, Group
from pyparsing import Keyword, Suppress, LineEnd
import sys
import os
import re

def studley(text):
    """
    Converts text containing underscores to studley caps. For example,
    studley("this_that_other") returns "thisThatOther". Text already in studley
    caps will not be modified.
    """
    return re.sub("_[a-zA-Z]", lambda match: match.group()[1:].upper(), text)
def camel(text):
    """
    Same as studley(...), but sets the first character to be upper-case.
    """
    text = studley(text)
    return text[0:1].upper() + text[1:]

# Maps protobuf types to the corresponding Java field types
java_type_map = {"string": "String", "fixed32": "Integer", "fixed64": "Long",
        "double": "Double", "bool": "Boolean"}
# Maps protobuf types to their expected wire type
wire_type_map = {"string": 2, "fixed32": 5, "fixed64": 1, "double": 1, "bool": 0}

class Field(object):
    def __init__(self, tokens):
        tokens = tokens.asList()
        self.mode = tokens[0] # required, optional, repeated
        self.type = tokens[1] # fixed32, bool, string, etc
        self.name = tokens[2] # name of the field
        self.number = int(tokens[3]) # field index in the protocol
        self.camel_name = camel(self.name)
        self.studley_name = studley(self.name)
        self.java_type = self.get_java_type()
        self.java_default = self.get_java_default()
        self.wire_type = wire_type_map.get(self.type, 2)
        self.component_type = java_type_map.get(self.type, self.type)
    
    def get_java_type(self):
        type = java_type_map.get(self.type, self.type)
        if self.mode == "repeated":
            type = "List<" + type + ">"
        return type
    
    def get_java_default(self):
        if self.mode == "repeated":
            return "new ArrayList<" + java_type_map.get(self.type, self.type) + ">()"
        return "null"
    
    def __repr__(self):
        return ("<Field " + self.name + " of type " + self.type + ", " + 
                self.mode + ", #" + str(self.number) + ">")
    __str__ = __repr__

class Message(object):
    def __init__(self, tokens):
        tokens = tokens.asList()
        self.name = tokens[0]
        self.fields = tokens[1]
    
    def __repr__(self):
        return "<Message " + self.name + " with fields " + repr(self.fields) + ">"
    __str__ = __repr__

comment = Literal("//") + Regex("[^\\n]*") + LineEnd()
field_type = Regex("[A-Za-z0-9_]+")
field_name = Regex("[A-Za-z0-9_]+")
field_number = Regex("[0-9]+")
message_field = ((Keyword("required") | Keyword("optional") | Keyword("repeated")) 
        + field_type + field_name + Suppress("=") + field_number + Suppress(";"))
message_field.setParseAction(Field)
message_name = Regex("[A-Za-z_]+")
message_spec = (Suppress(Keyword("message")) + message_name + Suppress("{") + 
         Group(ZeroOrMore(message_field)) + Suppress("}"))
message_spec.setParseAction(Message)
option_spec = (Suppress(Keyword("option")) + Regex("[a-z_]+") + Suppress("=") + 
        Regex('"[^"]*"') + Suppress(";"))
option_spec.setParseAction(lambda tokens: (tokens[0], tokens[1][1:-1]))
option_list = ZeroOrMore(option_spec)
option_list.setParseAction(lambda tokens: dict(tokens.asList()))
message_list = Group(ZeroOrMore(message_spec))
proto_file = (option_list + message_list).ignore(comment)

if len(sys.argv) < 3:
    print "usage: simpleproto some_file.proto outputfolder"
    print "The output will be placed in a folder within outputfolder"
    print "appropriate for the package specified in the file's java_package"
    print "option, in the file specified by java_outer_classname."

options, messages = proto_file.parseFile(sys.argv[1], parseAll=True).asList()
output_class = options["java_outer_classname"]
output_package = options["java_package"]
output_file_path = os.path.join(sys.argv[2], output_package.replace(
        ".", os.path.sep), output_class + ".java")
output_file = open(output_file_path, "w")

default_printf_args = {"output_class": output_class, "output_package":
        output_package}

def write(text, **kwargs):
    args = {}
    args.update(default_printf_args)
    args.update(kwargs)
    result = text % args
    result = re.sub("[\n]+", "\n", result)
    while result.startswith("\n"):
        result = result[1:]
    output_file.write(result)

write("""
// Auto-generated by Alexander Boyd (a.k.a. javawizard, jcp)'s simpleproto.py

package %(output_package)s;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.io.IOException;

public class %(output_class)s {
public static interface GeneratedMessage<E extends GeneratedMessage<E>> 
{public byte[] serialize();public E deserialize(byte[] bytes);
public <T> T get(String name);public void set(String name, Object value);
public Class<?> getInstanceClassForField(String fieldName);
}
protected static int readVarint(DataInputStream in)throws IOException{
    int value = 0;
    int i;
    int shift = 0;
    while(((i = in.read())&0x80)>0){
        value |= (i&0x7F)<<(shift*7);
        shift += 1;
    }
    value |= i<<(shift*7);
    return value;
}
protected static void writeVarint(DataOutputStream out, int value)throws IOException{
    if(value<0)throw new RuntimeException("Negative varints are not allowed.");
    while(value>0){
        int i = value&0x7F;
        value >>= 7;
        if(value>0)
            i |= 0x80;
        out.write(i);
    }
}
protected static byte[] reverseBytes(byte[] in){
    byte[] out = new byte[in.length];
    for(int i = 0; i < in.length; i++){
        out[(in.length-i)-1] = in[i];
    }
    return out;
}
protected static byte[] readReversedBytes(DataInputStream in, int length)throws IOException{
    byte[] bytes = new byte[length];
    in.readFully(bytes);
    return reverseBytes(bytes);
}
protected static void writeReversedBytes(DataOutputStream out, byte[] bytes)throws IOException{
    bytes = reverseBytes(bytes);
    out.write(bytes);
}
protected static DataInputStream readReversedToStream(DataInputStream in, int length)throws IOException{
    return new DataInputStream(new ByteArrayInputStream(readReversedBytes(in, length)));
}
protected static int readInt(DataInputStream in)throws IOException{
    return readReversedToStream(in, 4).readInt();
}
protected static long readLong(DataInputStream in)throws IOException{
    return readReversedToStream(in, 8).readLong();
}
protected static double readDouble(DataInputStream in)throws IOException{
    return readReversedToStream(in, 8).readDouble();
}
protected static void writeInt(DataOutputStream out, int value)throws IOException{
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    new DataOutputStream(baos).writeInt(value);
    writeReversedBytes(out, baos.toByteArray());
}
protected static void writeLong(DataOutputStream out, long value)throws IOException{
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    new DataOutputStream(baos).writeLong(value);
    writeReversedBytes(out, baos.toByteArray());
}
protected static void writeDouble(DataOutputStream out, double value)throws IOException{
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    new DataOutputStream(baos).writeDouble(value);
    writeReversedBytes(out, baos.toByteArray());
}
protected static void writeFieldNumber(DataOutputStream out, int fieldNumber, int wireType)throws IOException{
    writeVarint(out, (fieldNumber << 3) | wireType);
}
protected static int readFieldNumber(DataInputStream in)throws IOException{
    return readVarint(in) >> 3;
}
protected static byte[] readVarBytes(DataInputStream in)throws IOException{
    byte[] bytes = new byte[readVarint(in)];
    in.readFully(bytes);
    return bytes;
}
protected static String readString(DataInputStream in)throws IOException{
    return new String(readVarBytes(in));
}
protected static void writeVarBytes(DataOutputStream out, byte[] bytes)throws IOException{
    writeVarint(out, bytes.length);
    out.write(bytes);
}
protected static void writeString(DataOutputStream out, String string)throws IOException{
    writeVarBytes(out, string.getBytes());
}
""")

for message in messages:
    write("""
    public static class %(name)s implements GeneratedMessage<%(name)s> { 
    """, name=message.name)
    for field in message.fields:
        write("""
        public %(type)s %(name)s = %(default)s;
        """, type=field.java_type, name=field.studley_name, default=field.java_default)
    for field in message.fields:
        write("""
        public %(message_class)s set%(camel_name)s(%(type)s %(name)s){
            this.%(name)s = %(name)s;
            return this;
        }
        """, message_class=message.name, camel_name=field.camel_name,
             type=field.java_type, name=field.studley_name)
    write("public void checkRequired(){")
    for field in message.fields:
        if field.mode != "required":
            continue;
        write("""if(this.%(name)s == null)
        throw new RuntimeException("Field '%(name)s' (protobuf message field "
        + "'%(protoname)s') is required as per the protobuf definition "
        + "but no value for this field seems to be supplied.");
        """, name=field.studley_name, protoname=field.name)
    write("""
    }void readFrom(DataInputStream in)throws IOException{
        while(in.available() > 0){
            int fieldNumber = readFieldNumber(in);
    """)
    for field in message.fields:
        write("if(fieldNumber == %(number)s){%(ctype)s newValue =\n",
                number=field.number, ctype=field.component_type)
        if field.type == "fixed32":
            write("readInt(in);")
        elif field.type == "fixed64":
            write("readLong(in);")
        elif field.type == "double":
            write("readDouble(in);")
        elif field.type == "string":
            write("readString(in);")
        elif field.type == "bool":
            write("in.readBoolean();")
        else:
            write("""
            new %(ctype)s();
            byte[] bytes = readVarBytes(in);
            newValue.deserialize(bytes);
            """, type=field.java_type, ctype=field.component_type)
        if field.mode == "repeated":
            write("this.%(name)s.add(newValue);", name=field.studley_name)
        else:
            write("this.%(name)s=newValue;", name=field.studley_name)
        write("}else\n")
    write("""
            {throw new RuntimeException("Invalid field received: " + fieldNumber);}
        }checkRequired();
    }
    """)
    write("void writeTo(DataOutputStream out)throws IOException{")
    for field in message.fields:
        write("if(this.%(name)s != null){", name=field.studley_name)
        if field.mode == "repeated":
            write("for(%(ctype)s currentValue:%(name)s){",
                    ctype=field.component_type, name=field.studley_name)
        else:
            write("%(ctype)s currentValue = %(name)s;",
                    ctype=field.component_type, name=field.studley_name)
        write("writeFieldNumber(out, %(number)s, %(wire)s);",
                number=field.number, wire=field.wire_type)
        if field.type == "fixed32":
            write("writeInt(out, currentValue);")
        elif field.type == "fixed64":
            write("writeLong(out, currentValue);")
        elif field.type == "double":
            write("writeDouble(out, currentValue);")
        elif field.type == "string":
            write("writeString(out, currentValue);")
        elif field.type == "bool":
            write("out.writeBoolean(currentValue);")
        else:
            write("""
            byte[] bytes = currentValue.serialize();
            writeVarBytes(out, bytes);
            """)
        if field.mode == "repeated":
            write("}")
        write("}")
    write("""
    }
    public %(message_type)s deserialize(byte[] bytes){
        try{readFrom(new DataInputStream(new ByteArrayInputStream(bytes)));}
        catch(IOException e){throw new RuntimeException(e);}return this;
    }
    public byte[] serialize(){
        try{
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            writeTo(new DataOutputStream(out));
            return out.toByteArray();
        }catch(IOException e){throw new RuntimeException(e);}
    }
    private static Map<String,Class<?>> _typeMap = new HashMap<String,Class<?>>();
    public static Set<String> getFieldNames(){
        return _typeMap.keySet();
    }
    public static Class<?> getClassForField(String name){
        return _typeMap.get(name);
    }
    public Class<?> getInstanceClassForField(String name){
        return getClassForField(name);
    }
    public static List<String> getFieldNamesForPrefix(String prefix){
        ArrayList<String> list = new ArrayList<String>();
        for(String name : getFieldNames()){
            if(name.startsWith(prefix))
                list.add(name);
        }
        return list;
    }
    public void set(String name, Object value){
        try{
            getClass().getField(name).set(this, value);
        }catch(Exception e){throw new RuntimeException(e);}
    }
    @SuppressWarnings("unchecked")
    public <T> T get(String name){
        try{
            return (T) getClass().getField(name).get(this);
        }catch(Exception e){throw new RuntimeException(e);}
    }
    static{
    """, message_type=message.name)
    for field in message.fields:
        write("_typeMap.put(\"%(name)s\", %(ctype)s.class);",
                name=field.studley_name, ctype=field.component_type)
    write("}}")

write("}\n")

output_file.close()

































"""
A collection of utilities used by lots of different programs and libraries in
the AFN project that don't really fit into just one of those programs/libraries.
Most (or maybe all... I'm not sure at the moment...) of these were written by
Alexander Boyd.
"""

from traceback import print_exc as _print_exc
import time
from threading import Thread
from functools import update_wrapper

def slicer(length, start=None, stop=None, step=None):
    """
    A generator that yields the indexes (in the correct order) that the
    specified Python list slice, evaluated over a sequence of the specified
    length, would produce.
    """
    start, stop, step = slice(start, stop, step).indices(length)
    if(step > 0):
        while start < stop:
            yield start
            start += step
    else:
        while start > stop:
            yield start
            start += step


def field(function):
    name = function.__name__
    def get_function(self):
        return getattr(self, "_p_" + name)
    def set_function(self, value):
        setattr(self, "_p_" + name, value)
    p = property(get_function, set_function, doc=function.__doc__)
    return p


def full_name(some_class):
    if not isinstance(some_class, type): # This is an instance of a class, not
        # a class itself, so we need to get this object's class
        some_class = type(some_class)
    return some_class.__module__ + "." + some_class.__name__


class BlankObject(object):
    """
    A blank object that allows arbitrary assignment to its instances'
    attributes.
    """
    pass


def cast(instance, *types):
    """
    Raises an exception if isinstance(instance, types) returns False. This is
    useful in, for example, Autobus interfaces to ensure that objects passed
    in by a client are of the correct type.
    """
    if not isinstance(instance, types):
        raise Exception("The specified object's type is " + str(type(instance))
                + ", but it needs to be one of " + str(types))


class NoExceptions(object):
    """
    A with-statement context manager that suppresses and silently discards all
    exceptions thrown from within it. You'll usually want to use the single
    instance of this class, afn.utils.no_exceptions.
    """
    def __enter__(self):
        pass
    
    def __exit__(self, *args):
        return True

no_exceptions = NoExceptions()


class Suppress(object):
    """
    A context manager that suppresses exceptions of a particular type. Such
    exceptions, when thrown from within the block using this context manager,
    will be silently discarded.
    """
    def __init__(self, suppress_type):
        self.suppress_type = suppress_type
    
    def __enter__(self):
        pass
    
    def __exit__(self, exception_type, *args):
        if exception_type:
            return issubclass(exception_type, self.suppress_type)


class Consume(object):
    """
    A context manager that accepts an exception type and a function as
    constructor arguments. If an exception is thrown from within a with
    statement using this context manager, and the specified function is
    actually a function (or some other type of callable), then the function
    is called, passing in the exception, and the exception is suppressed. If
    the function is not callable, the exception is propagated upward as normal.
    
    If the consume parameter is False, this context manager does nothing. This
    is used in a few cases in Autobus where whether or not a particular piece
    of code should be run under a Consume context manager is dependent on a
    particular argument to the function running the code; most of the functions
    in Autobus that accept a parameter named "safe" are implemented this way,
    with the value of this safe parameter being passed as the consume parameter.
    """
    def __init__(self, exception_type, function, consume):
        self.exception_type = exception_type
        self.function = function
        self.consume = consume
    
    def __enter__(self):
        pass
    
    def __exit__(self, t, v, tb):
        if not self.consume: # We're not supposed to do anything
            return
        if not t: # No exception was thrown
            return
        if not issubclass(t, self.exception_type): # Not the type of exception
            # we're looking for
            return
        if callable(self.function):
            self.function(v)
            return True


class PrintExceptions(object):
    """
    Similar to NoExceptions and no_exceptions, but prints (with
    traceback.print_exc) any exceptions that it suppresses.
    """
    def __enter__(self):
        pass
    
    def __exit__(self, t, v, tb):
        if tb:
            _print_exc()
        return True

print_exceptions = PrintExceptions()


def filter_dict(input, rule_map):
    """
    Creates a new dictionary containing one key for each key that's present in
    both input and rule_map, which should both be dictionaries. The key
    present in the output dictionary will be the value of the corresponding
    entry in rule_map.
    
    In other words, this copies input, removes all keys not present in
    rule_map, then renames them to be the values of the matching keys in
    rule_map.
    """
    new = {}
    for old_key, new_key in rule_map.items():
        if old_key in input:
            new[new_key] = input[old_key]
    return new


def at(seconds, function):
    """
    Starts a new thread that will run the specified function after the
    specified number of seconds. The function will only be run once; after
    running the function, the thread will die.
    """
    def run():
        time.sleep(seconds)
        function()
    Thread(name="afn.utils.at", target=run).start()


def print_on_fail(function):
    """
    A decorator that can be used thus:
    
    @print_on_fail
    def example(...):
        ...
    
    to call print_exc whenever an uncaught exception gets thrown out of the
    function. The exception will continue to propagate after the stack trace
    has been printed.
    """
    def wrapper(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except:
            _print_exc()
            raise
    update_wrapper(wrapper, function)
    return wrapper


def rotations(sequence):
    """
    rotations("ABC") -> ["ABC", "BCA", "CAB"]
    """
    return [(sequence[i:] + sequence[:i]) for i in range(len(sequence))]


def wrap(function):
    """
    Returns a newly-created function that simply calls the specified one with
    the specified arguments. The returned function behaves, when called, as if
    it was the function originally passed into the wrap function.
    
    The reason that this is useful is that accessing a bound method of a
    particular object actually returns a different bound method object each
    time, so tracking these in a list of, say, listeners doesn't work. The wrap
    function can be used to wrap a bound method in a function that won't
    change (and that, furthermore, is always hashable, whereas bound methods
    are only hashable if their underlying objects are also hashable), and the
    wrapper can then be used as the listener.
    """
    return lambda *args, **kwargs: function(*args, **kwargs)


def get_path(object, path):
    if isinstance(path, basestring):
        components = path.split(".")
    for component in components:
        object = getattr(object, component)
    return object

def print_args(*args, **kwargs):
    print args, kwargs


































from autobus2 import Bus, wait_for_interrupt

class Service(object):
    def set(self, new_state):
        state_object.set_value(new_state)
        
def main():
    global state_object
    with Bus() as bus:
        service = bus.create_service({"type": "flipflop"}, active=False, use_py_object=Service())
        state_object = service.create_object("state", 1, "The current state.")
        service.activate()
        wait_for_interrupt()




from concurrent import synchronized
from threading import RLock
from functools import partial
from utils import print_on_fail

global_lock = RLock()
locked = synchronized(global_lock)

class Connection(object):
    @locked
    def __init__(self, protocol, event_function, close_function, features,
            widget_constructors={}, error_function=lambda fatal, packet: None):
        """
        Creates a new connection. TODO: document this more later.
        
        WARNING: If you're using librtkinter or any Tkinter UI with this,
        DO NOT pass tk.after_idle as the event function. This will cause
        sporadic deadlocks, which won't be noticeable unless the server sends
        a whole bunch of UI updates all at once. Instead, have event_function
        be the put function of a queue, and then write a function that
        registers itself with tk.after to run every, say, 200ms and runs all
        the events in the queue.
        http://me.opengroove.org/2011/01/threading-locks-and-tkinters-afteridle.html
        (which is a blog post I wrote on this issue) provides more info as to
        why these deadlocks occur.
        """
        self.protocol = protocol
        self.schedule = event_function
        self.close_function = close_function
        self.error_function = error_function
        self.features = features
        self.widget_constructors = widget_constructors.copy()
        self.started = False
        self.handshake_finished = False
        self.closed = False
        self.widget_map = {}
        self.toplevels = []
        # We shouldn't need a pre-start list of messages like the server does
        # since the server's not supposed to send us anything until after we
        # send it the handshake packet. I might add such a list later if a lot
        # of non-conformant servers (the type that want to accept every client
        # and so send the accept message immediately after a connect) start
        # turning up on the internet.
        protocol.protocol_init(self)
    
    @locked
    def start(self):
        if self.started:
            raise Exception("You can't call Connection.start twice.")
        self.started = True
        self.protocol.protocol_start()
        self.send({"action": "connect", "features": self.features})
    
    @locked
    def send(self, packet):
        self.protocol.protocol_send(packet)
    
    @locked
    def send_event(self, id, name, user, *args):
        """
        Sends an event to the server. id is the id of the widget that the
        event occurred on. name is the name of the event. user is whether or
        not the event was triggered by a user interacting with the interface
        (as opposed to a widget function being called). *args are the arguments
        to be sent with the event.
        """
        self.send({"action":"event", "id":id, "name":name,
                "user":user, "args":args})
    
    @locked
    def send_set_state(self, id, properties):
        """
        """
        self.send({"action":"set_state", "id":id, "properties": properties})
    
    @locked
    def fatal_error(self, message):
        # TODO: Right now, errors that are most likely the fault of the
        # programmer and errors that result from the client messing up are
        # both printed by this function. Perhaps split them out so the
        # application developer knows what to look for to know if they have
        # potential problems with their application.
        print "FATAL CONNECTION ERROR: " + message
        self.send({"action": "drop", "text": message})
        self.close()
    
    @locked
    def protocol_receive(self, packet):
        if not self.started:
            print ("HUGE MAJOR PROBLEM: Packet received before handshake "
                    "sent; this packet is being ignored, which is most "
                    "likely going to result in a corrupted datastream.")
            return
        if self.closed:
            return
        if packet["action"] == "error":
            print "SERVER REPORTED AN ERROR: " + packet["text"]
            self.error_function(False, packet)
            return
        if packet["action"] == "drop":
            print "SERVER REPORTED A FATAL ERROR: " + packet["text"]
            self.error_function(True, packet)
            self.close()
            return
        if not self.handshake_finished:
            if packet["action"] != "accept":
                self.fatal_error("First message must be an accept message")
                return
            self.server_features = packet["features"]
            self.handshake_finished = True
            return
        action = packet["action"]
        if action == "create":#id,parent,type,index,p_widget,p_layout
            self.schedule(partial(self.on_create, packet["id"],
                    packet.get("parent", None), packet["type"],
                    packet.get("index", None), packet.get("p_widget", {}),
                    packet.get("p_layout", {})))
        elif action == "destroy":
            self.schedule(partial(self.on_destroy, packet["id"]))
        elif action == "reorder":
            self.schedule(partial(self.on_reorder, packet))
        elif action == "set_widget":
            self.schedule(partial(self.on_set_widget, packet["id"],
                    packet["properties"]))
        elif action == "set_layout":
            self.schedule(partial(self.on_set_layout, packet["id"],
                    packet["properties"]))
        elif action == "call":
            self.schedule(partial(self.on_call, packet["id"], packet["name"],
                    packet["args"]))
        else:
            print "Ignoring message with action " + action
    
    @locked
    def protocol_connection_lost(self):
        self.close()
    
    def tri_call(self, widget, name, *args):
        if widget.parent and hasattr(widget.parent, "pre_" + name):
            getattr(widget.parent, "pre_" + name)(widget, *args)
        if hasattr(widget, name):
            getattr(widget, name)(*args)
        if widget.parent and hasattr(widget.parent, "post_" + name):
            getattr(widget.parent, "post_" + name)(widget, *args)
    
    @locked
    def close(self):
        if self.closed:
            return
        self.closed = True
        self.protocol.protocol_close()
        self.schedule(partial(self.on_close))
    
    @print_on_fail
    def on_create(self, id, parent_id, type, index, widget_properties,
            layout_properties):
        if type not in self.widget_constructors:
            self.fatal_error("Unsupported widget type: " + type)
            return
        if parent_id is not None and parent_id not in self.widget_map:
            self.fatal_error("Trying to add child " + str(id) + " to parent "
                    + str(parent_id) + ", which does not exist.")
            return
        if id in self.widget_map:
            self.fatal_error("Trying to add a widget with id " + str(id) + 
                    " but there's already a non-destroyed widget with that id")
            return
        constructor = self.widget_constructors[type]
        parent_widget = self.widget_map[parent_id] if parent_id is not None else None
        widget = constructor(self, id, parent_widget, type, index,
                widget_properties.copy(), layout_properties.copy())
        self.widget_map[id] = widget
        if parent_widget is None:
            self.toplevels.append(widget)
        else:
            parent_widget.children[index:index] = [widget]
        self.tri_call(widget, "setup")
    
    @print_on_fail
    def on_destroy(self, id):
        widget = self.widget_map[id]
        for child in widget.children[:]:
            self.on_destroy(child.id)
        self.tri_call(widget, "destroy")
        del self.widget_map[id]
        if widget.parent:
            widget.parent.children.remove(widget)
            widget.parent = None # Unlink to try and minimize reference cycles
        else:
            self.toplevels.remove(widget)
    
    @print_on_fail
    def on_set_widget(self, id, properties):
        widget = self.widget_map[id]
        widget.widget_properties.update(properties)
        widget.update_widget(properties.copy())
    
    @print_on_fail
    def on_set_layout(self, id, properties):
        widget = self.widget_map[id]
        widget.layout_properties.update(properties)
        if widget.parent:
            widget.parent.update_layout(widget, properties.copy())
    
    @print_on_fail
    def on_call(self, id, name, args):
        widget = self.widget_map[id]
        widget.call(name, args)
    
    @print_on_fail
    def on_close(self):
        for toplevel in self.toplevels[:]:
            self.on_destroy(toplevel.id)
        self.close_function()
    

class Widget(object):
    def __init__(self, connection, id, parent, type, index,
            widget_properties, layout_properties):
        self.connection = connection
        self.id = id
        self.parent = parent
        self.type = type
        self.index = index
        self.widget_properties = widget_properties
        self.layout_properties = layout_properties
        self.children = []
    
    def send_event(self, name, user, *args):
        self.connection.send_event(self.id, name, user, *args)
    
    def send_set_state(self, properties={}, **kwargs):
        p = {}
        p.update(properties)
        p.update(kwargs)
        self.connection.send_set_state(self.id, p)
    
    def call(self, name, args):
        self.connection.fatal_error("Widget " + str(self.id) + " of type "
                + self.type + " does not provide any calls, but the server "
                "just tried to perform a call.")

#class Dispatcher(object):
#    """
#    This class specifies the dispatcher interface. Dispatchers are objects
#    passed into Connection instances. They implement the actual display
#    function of an RTK client.
#    
#    Dispatchers are not required to subclass from this class, but they can;
#    This class exists primarily to document the functions a dispatcher must
#    have. All functions on this class raise exceptions indicating that they
#    have not been implemented.
#    """
#    def dispatcher_init(self, connection):
#        """
#        Initializes this dispatcher. The connection this dispatcher was passed
#        to is passed as the argument.
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_ready(self):
#        """
#        Called once the connection has been started and the handshake has been
#        completed. The dispatcher should expect widget function calls to start
#        arriving shortly after this function is called. No other functions
#        besides dispatcher_init and dispatcher_close will be called before
#        this function is called.
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_close(self):
#        """
#        Called when the connection has been shut down. The dispatcher should
#        destroy all widgets that have not yet been destroyed. This will be the
#        last function called on the dispatcher.
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_received_error(self, error):
#        """
#        Called when an error is sent by the server. This is not a fatal error;
#        communications will continue as normal after the error. The dispatcher
#        should inform the user of the error, which is passed into this
#        function as a dictionary with one key, text, which is the text of the
#        error.
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_received_drop(self, error):
#        """
#        Same as dispatcher_received_error, but the error is fatal;
#        dispatcher_close will be called shortly after this function returns. 
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_create(self, id, parent, type, index, widget_properties,
#            layout_properties):
#        """
#        Called to create a widget. id is the integer id that is to be assigned
#        to the widget. parent is the id of the parent widget, or None if this
#        is to be a toplevel. type is the type of the widget ("Window", "VBox",
#        "Button", "Label", etc). index is the index within the widget's parent
#        at which the widget is to be created (which will range from 0 to the
#        number of widgets the parent currently contains, inclusive); this is
#        unspecified in the case of a toplevel. widget_properties is a
#        dictionary of the properties the widget is to be created with.
#        layout_properties is a dictionary of the layout properties the widget
#        is to be created with.
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_destroy(self, id):
#        """
#        Called to destroy a widget. id is the integer id of the widget to
#        destroy.
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_set_widget(self, id, properties):
#        """
#        Called to change a widget's widget properties. id is the integer id of
#        the widget. properties is a dictionary of properties to change.
#        """
#        raise Exception("Not implemented")
#    
#    def dispatcher_set_layout(self, id, properties):
#        """
#        Same as dispatcher_set_widget, but sets layout properties instead of
#        widget properties.
#        """
#        raise Exception("Not implemented")
    





































from afn.utils import singleton

# _NO_VALUE = singleton.Singleton("nextgen.keys.observing._NO_VALUE")

class Observable(object):
    def _get_current_values(self, value_list):
        pass
    
    def _init_observable(self):
        try:
            self._observers
        except AttributeError:
            self._observers = {}
            self._recursive_observer_count = 0
            # We're defining this here instead of as a normal class method
            # because bound methods (i.e. some_instance.some_method) can change
            # ids, but functions defined like this can't, and we need the same
            # id so that things we observe know when we're removing ourselves
            # as an observer
            def _recursive_listener_function(path, hierarchy, changes, context):
                for registration in self._observers.values():
                    if registration["recursive"]:
                        registration["observer"](self, changes)
            self._recursive_listener = _recursive_listener_function
    
    def observe(self, observer, recursive=False, context=None):
        self._init_observable()
        if id(observer) in self._observers:
            raise ValueError("The specified observer has already been added to "
                             "this object.")
        registration = {"observer": observer, "recursive": recursive, "context": context}
        self._observers[id(observer)] = registration
        if recursive:
            self._recursive_observer_count += 1
            if self._recursive_observer_count == 1: # This was the first
                # recursive observer to be added, so we need to go actually
                # listen to all of our current children for changes
                values = []
                self._get_current_values(values)
                for value in values:
                    if isinstance(value, Observable):
                        value.observe(self._recursive_listener, True)
    
    def unobserve(self, observer):
        self._init_observable()
        if id(observer) not in self._observers:
            raise ValueError("The specified observer has not been added as to "
                             "this object.")
        registration = self._observers[id(observer)]
        # Remove the actual registration
        del self._observers[id(observer)]
        # Then see if it was recursive
        if registration["recursive"]:
            self._recursive_observer_count -= 1
            if self._recursive_observer_count == 0: # This was the last
                # recursive observer to be removed, so we're going to go remove
                # the listeners we've placed on all of our children 
                values = []
                self._get_current_values(values)
                for value in values:
                    if isinstance(value, Observable):
                        value.unobserve(self._recursive_listener)
    
    def _notify_changed(self, changes):
        if len(changes) == 0:
            return
        self._init_observable()
        path = []
        hierarchy = [self]
        # See if we're recursively listening at present
        if self._recursive_observer_count > 0:
            # If we are, see if this change adds or removes any values
            for change in changes:
                if hasattr(change, "old"):
                    # A value was removed. If it's observable, stop listening
                    # for changes.
                    if isinstance(change.old, Observable):
                        change.old.unobserve(self._recursive_listener)
                if hasattr(change, "new"):
                    # A value was added. If it's observable, start listening
                    # for changes.
                    if isinstance(change.new, Observable):
                        change.new.observe(self._recursive_listener, True)
        # Now we need to notify all of our own observers about changes.
        for registration in self._observers.itervalues():
            registration["observer"](path, hierarchy, changes, registration["context"])











































__doc__ = """\
This module is a simple protocol documentation generator. It takes an input
file and converts it to an HTML file. The input file is line-based.

The file is divided into sections, each representing a particular item that
can be sent across the wire. Each section is further divided into a list of
items, each of which has a type and a description. The type is either a
certain number of bytes, the name of another section, or a list of the names
of another section. The description of an item is just information on what
that item is. An item can also be marked as repeating, which means the item
can appear multiple times. The description will specify whether it can appear
zero times or if it has to appear at least once, and the description will
usually explain what governs how many times it can appear. A common practice
is to have a single byte specifying how many instances of some particular item
appear, and then have that repeating item appear immediately following.

So, a section is started with a single word on a single line. This specifies
the name of the section. Individual items within a section are specified as
item: value, where item is a number, another section name, or a space-separated
list of other section names, and value is the textual description of the item.

An item, whatever the type, can also be suffixed with "...", which means that
the item is a repeating item, as discussed above.

Item values are interpreted as HTML.

Any text in the file before the first section (which is a single word on a
single line) is placed at the top of the file before the sections. 
"""




class Multimap(dict):
    def __init__(self, on_create=None, on_delete=None, on_before_create=None,
            on_before_delete=None):
        self.on_create = on_create
        self.on_delete = on_delete
        self.on_before_create = on_before_create
        self.on_before_delete = on_before_delete
    
    def add(self, key, value):
        created = False
        if not key in self:
            if self.on_before_create:
                self.on_before_create()
            self[key] = []
            created = True
        self[key].append(value)
        if created:
            if self.on_create is not None:
                self.on_create(key)
    
    def remove(self, key, value):
        if self[key] == [value]:
            if self.on_before_delete:
                self.on_before_delete()
        self[key].remove(value)
        if not self[key]:
            del self[key]
            if self.on_delete is not None:
                self.on_delete(key)
    
    # TODO: add more stuff to this class



from abc import ABCMeta as ABC, abstractmethod as abstract
from afn.utils import singleton

_NO_VALUE = singleton.Singleton("nextgen.keys2.observing._NO_VALUE")

class KeyObservable(object):
    __metaclass__ = ABC
    
    def observe_path(self, observer, path, ):
        raise NotImplementedError


class ObservableObject(KeyObservable):
    def __setattr__(self, name, value):
        try:
            old = object.__getattribute__(self, name, value)
        except AttributeError: # Attribute doesn't exist yet
            old = _NO_VALUE
        object.__setattr__(self, name, value)
        if old is _NO_VALUE:
            self._notify_key_added(name, value)
        else:
            self._notify_key_changed(name, old, value)
    
    def __delattr__(self, name, value):
        old = object.__getattribute__(self, name, value)
        self._notify_key_removed(name, old)
    
    def set_value_for_key(self, key, value):
        setattr(self, key, value)
    
    def get_value_for_key(self, key):
        return getattr(self, key)







try:
    import Tix as tkinter
    tix_support = True
except ImportError:
    tix_support = False
    import Tkinter as tkinter
import librtkclient
from Queue import Queue, Empty
from traceback import print_exc
from utils import filter_dict
import tkFont

"""
This module is an RTK viewer library that uses Tkinter to show the
application.

The simplest way to start a viewer displaying an RTK application located at
localhost port 6785 would be something like this:

import librtkinter
from librtk.protocols import ThreadedProtocol
from socket import socket
s = socket()
s.connect(("localhost", 6785))
connection, tk_master = librtkinter.start_connection(ThreadedProtocol(s))
tk_master.mainloop()
"""

#class TkinterDispatcher(Dispatcher):
#    """
#    This is the Tkinter dispatcher implementation.
#    """
#    def __init__(self, master, event_function, close_function,
#            widget_constructors={}, use_default_widgets=True):
#        """
#        Creates a new Tkinter dispatcher. master is a Tk instance under which
#        widgets and windows will be created. event_function is a function that
#        will be called whenever this class needs a function run on the Tkinter
#        event loop. close_function is a function that will be called when the
#        dispatcher is informed that the connection has been lost.
#        widget_constructors is a map of widget names to functions that can be
#        used to construct instances of those widgets. use_default_widgets
#        can be true to automatically add the global default_widgets to the
#        dictionary; widgets specified in widget_constructors will override
#        those specified in default_widgets.
#        """
#        self.master = master
#        self.schedule = event_function
#        self.close_function = close_function
#        self.widget_constructors = {}
#        # Update with defaults first, then user-specified so that
#        # user-specified constructors override the buit-in ones
#        if use_default_widgets:
#            self.widget_constructors.update(default_widgets)
#        self.widget_constructors.update(widget_constructors)
#        self.widget_map = {} # Map of widget ids to widgets
#        self.toplevels = [] # List of toplevel widget objects
#    
#    def fatal_error(self, text):
#        self.connection.fatal_error(text)
#
#    def dispatcher_init(self, connection):
#        self.connection = connection
#    
#    def dispatcher_start(self):
#        pass
#
#    def dispatcher_ready(self):
#        pass
#
#    def dispatcher_close(self):
#        self.schedule(partial(self.close))
#
#    def dispatcher_received_error(self, error):
#        self.schedule(partial(self.received_error, error))
#
#    def dispatcher_received_drop(self, error):
#        self.schedule(partial(self.received_drop, error))
#
#    def dispatcher_create(self, id, parent, type, index, widget_properties,
#            layout_properties):
#        self.schedule(partial(self.create, id, parent, type, index,
#                widget_properties, layout_properties))
#
#    def dispatcher_destroy(self, id):
#        self.schedule(partial(self.destroy, id))
#
#    def dispatcher_set_widget(self, id, properties):
#        self.schedule(partial(self.set_widget, id, properties))
#
#    def dispatcher_set_layout(self, id, properties):
#        self.schedule(partial(self.set_layout, id, properties))
#    
#    def received_error(self, error):
#        print "ERROR RECEIVED: " + error["text"]
#    
#    def received_drop(self, error):
#        print "FATAL ERROR RECEIVED: " + error["text"]
#    
#    def create(self, id, parent, type, index, widget_properties,
#            layout_properties):
#        if type not in self.widget_constructors:
#            self.fatal_error("Unsupported widget type: " + type)
#            return
#        if parent is not None and parent not in self.widget_map:
#            self.fatal_error("Trying to add child " + str(id) + " to parent "
#                    + str(parent) + ", which does not exist.")
#            return
#        if id in self.widget_map:
#            self.fatal_error("Trying to add a widget with id " + str(id) +
#                    " but there's already a non-destroyed widget with that id")
#        constructor = self.widget_constructors[type]
#        widget = constructor()
#        widget.dispatcher = self
#        widget.id = id
#        widget.parent = self.widget_map[parent] if parent is not None else None
#        widget.type = type
#        widget.connection = self.connection
#        widget.children = []
#        widget.widget_properties = widget_properties.copy()
#        widget.layout_properties = layout_properties.copy()
#        widget.state_properties = {}
#        widget.send_event = partial(self.connection.send_event, id)
#        self.widget_map[id] = widget
#        if parent is None:
#            self.toplevels.append(widget)
#        else:
#            widget.parent.children[index:index] = [widget]
#        widget.setup()
#        if parent is not None:
#            widget.parent.setup_child(widget)
#    
#    def destroy(self, id):
#        widget = self.widget_map[id]
#        # FIXME: Tell the parent about the destroy, too
#        widget.destroy()
#        del self.widget_map[id]
#        # FIXME: remove from the list of toplevels
#    
#    def set_widget(self, id, properties):
#        widget = self.widget_map[id]
#        widget.widget_properties.update(properties)
#        widget.update_widget_properties(properties)
#    
#    def set_layout(self, id, properties):
#        widget = self.widget_map[id]
#        widget.layout_properties.update(properties)
#        if widget.parent is not None:
#            widget.parent.update_layout_properties(widget, properties)
#    
#    def close(self):
#        for toplevel in self.toplevels[:]:
#            toplevel.destroy()
#        self.close_function()
#        # TODO: consider whether the children should be destroyed first


class Widget(librtkclient.Widget):
    tk_use_font = False
    
    def post_setup(self, child):
        # This will always be overridden to pack the widget in its parent,
        # so we'll use it to send back an error.
        self.connection.fatal_error("Widget type " + type(self)
                + " is not a container, but you just tried to add a child "
                "to it.")
    
    def destroy(self):
        self.widget.destroy()
    
    def update_widget(self, properties=None):
        """
        Default implementation of update_widget. This looks for a field on the
        widget class named tk_fields. If it's present, it should be a list of
        fields, with each field represented by a list containing either two
        or three items: [rtk_field_name, tkinter_field_name, default]. The
        default, if present, will be substituted for the value to set on the
        field if bool(value_to_be_set) == False.
        
        This also sets up font information if the widget class has a field
        tk_use_font = True.
        """
        if hasattr(self, "tk_fields"):
            for field in self.tk_fields:
                # Get the name of the field and its corresponding TK name
                rtk_name, tkinter_name = field[0:2]
                # Now we se if this request is supposed to change this field
                if rtk_name in properties:
                    # Looks like it is. We'll set the field's value if it has
                    # a true value or if there's no default to look to in case
                    # the specified value is a false value.
                    if properties[rtk_name] or len(field) <= 2:
                        # Yep, it's true! We'll set the value and we're done.
                        self.widget[tkinter_name] = properties[rtk_name]
                    else: # The value we're supposed to set this field to is
                        # a false value and the field has a default. We'll set
                        # the field to the default value.
                        self.widget[tkinter_name] = field[2]
        if self.tk_use_font:
            if "font_size" in properties:
                self.update_font()
    
    def update_font(self):
        font_info = filter_dict(self.widget_properties, {"font_size": "size"})
        font_info = dict([(k, v) for k, v in font_info.items() if v])
        font = tkFont.Font(**font_info)
        self.widget["font"] = font
    
    def update_layout(self, child, properties):
        pass
    
    @property
    def container(self):
        return self.widget


class Window(Widget):
    def setup(self):
        self.widget = tkinter.Toplevel(self.connection.tk_master)
        self.widget.protocol("WM_DELETE_WINDOW", self.window_close_request)
        if "title" in self.widget_properties:
            self.widget.title(self.widget_properties["title"])
    
    def post_setup(self, child):
        child.widget.pack(fill=tkinter.BOTH, expand=True)
    
    def update_widget(self, properties):
        if "title" in properties:
            self.widget.title(properties["title"])
    
    def window_close_request(self):
        self.send_event("close_request", True)


class Table(Widget):
    grid_rule_map = {"rowspan": "rowspan", "colspan": "columnspan",
            "row": "row", "col": "column", "pin": "sticky"}
     
    def setup(self):
        self.widget = tkinter.Frame(self.parent.container)
    
    def post_setup(self, child):
        child.widget.grid(**filter_dict(child.layout_properties,
                Table.grid_rule_map))
    
    def update_layout(self, child, properties):
        child.widget.grid(**filter_dict(properties, Table.grid_rule_map))


class Label(Widget):
    tk_fields = [["text", "text"]]
    tk_use_font = True
    
    def setup(self):
        self.widget = tkinter.Label(self.parent.container,
                text=self.widget_properties["text"])
        self.default_background = self.widget["bg"]
        if self.widget_properties["background"]:
            self.widget["bg"] = self.widget_properties["background"]
        if (self.widget_properties["font_size"] or
                self.widget_properties["font_family"]):
            self.update_font()
    
    def update_widget(self, properties):
        Widget.update_widget(self, properties)
        if "background" in properties:
            if properties["background"]:
                self.widget["bg"] = properties["background"]
            else:
                self.widget["bg"] = self.default_background

class VBox(Widget):
    def setup(self):
        self.widget = tkinter.Frame(self.parent.container)
    
    def post_setup(self, child):
        child.widget.pack(side=tkinter.TOP)


class HBox(Widget):
    def setup(self):
        self.widget = tkinter.Frame(self.parent.container)
    
    def post_setup(self, child):
        child.widget.pack(side=tkinter.LEFT)


class BorderPanel(Widget):
    tk_fields = [["border_width", "borderwidth"]]
    
    def setup(self):
        self.widget = tkinter.Frame(self.parent.container)
        self.widget["relief"] = tkinter.SOLID
        self.widget["borderwidth"] = (self.widget_properties["border_width"]
                if "border_width" in self.widget_properties else 1)
    
    def post_setup(self, child):
        child.widget.pack(fill=tkinter.BOTH, expand=True)


class Button(Widget):
    tk_fields = [["text", "text"]]
    
    def setup(self):
        self.widget = tkinter.Button(self.parent.container,
                text=self.widget_properties["text"],
                command=self.button_clicked)
    
    def button_clicked(self):
        self.send_event("clicked", True)


class TextBox(Widget):
    tk_fields = [["width", "width"], ["background", "bg", "#ffffff"]]
    tk_use_font = True
    
    def setup(self):
        self.var = tkinter.StringVar(self.connection.tk_master, "")
        self.widget = tkinter.Entry(self.parent.container,
                width=self.widget_properties["width"], textvariable=self.var,
                bg=self.widget_properties["background"] or "#ffffff")
        self.var.trace_variable("w", self.text_changed)
        self.send_set_state(text="")
        self.user_set = False
        if (self.widget_properties["font_size"] or
                self.widget_properties["font_family"]):
            self.update_font()
    
    def text_changed(self, *args):
        text = self.var.get()
        # FIXME: include user_set as the _user argument, or add something to
        # librtkclient to do this
        self.send_set_state(text=text)
        self.user_set = False
    
    def call(self, name, args):
        if name == "set_text":
            text, = args
            self.user_set = True
            self.var.set(text)
        else:
            self.connection.fatal_error("Invalid call " + name + " on TextBox")


widget_list = [Window, Label, VBox, HBox, BorderPanel, Button, Table, TextBox]
widget_set = dict([(w.__name__, w) for w in widget_list])
feature_set = ["widget:" + w for w in widget_set.keys()]

def start_connection(protocol):
    """
    Sets up a Tkinter viewer connected to the specified protocol instance. A
    toplevel Tk instance will be created and all the event processing stuff
    set up for it. A connection wrapping it will then be created and started,
    and this function will return a 2-tuple of the connection and the master
    Tk instance. This can be used to implement a simple RTK viewer like so:
    
    import socket, librtkinter
    from librtk.protocols import ThreadedProtocol
    s = socket.connect(("localhost", 6785))
    connection, tk = librtkinter.start_connection(ThreadedProtocol(s))
    tk.mainloop()
    
    
    """
    tk = tkinter.Tk()
    tk.withdraw()
    event_queue = Queue()
    connection = librtkclient.Connection(protocol, event_queue.put, tk.destroy,
            feature_set, widget_set)
    connection.tk_master = tk
    connection.start()
    def idle():
        try:
            event_queue.get(block=False)()
        except Empty:
            tk.after(100, idle)
        except:
            tk.after(100, idle)
            print_exc()
        else:
            tk.after(0, idle)
    tk.after(100, idle)
    return connection, tk






























# FIXME: convert this to Autobus 2

from autobus2 import Bus, wait_for_interrupt
from threading import Thread, RLock
from datetime import datetime
from time import sleep
import sys
from concurrent import synchronized
from utils import cast
from afn.utils import print_exceptions
from functools import partial

UP = 1
DOWN = 2
STOPPED = 3
STATES = [UP, DOWN, STOPPED]

lock = RLock()
time_since_startup = 0
startup_time = datetime.now()
timer_map = {} # Maps timer numbers to Timer instances
is_shut_down = False

class Timer(object):
    def __init__(self, number):
        """
        Creates a new timer. This does not add it to the timer map.
        """
        self.number = number
        self.time = 0
        self.state = STOPPED
        self.announce_interval = 0
        self.name = ""
        self.announce_count = 5
        self.announce_on_state_change = True
        self.is_beeping = False
    
    def get_absolute_time(self):
        """
        Calculates the absolute time remaining on this timer. If this timer's
        state is stopped, this returns self.time. If this timer's state is
        counting up, this returns time_since_startup - self.time. If this
        timer's state is counting down, this returns
        self.time - time_since_startup.
        """
        if self.state == STOPPED:
            return self.time
        if self.state == UP:
            return time_since_startup - self.time
        if self.state == DOWN:
            return self.time - time_since_startup
    
    def set_absolute_time(self, new_time):
        """
        Sets the absolute time on this timer to the specified value.
        """
        if self.state == STOPPED:
            self.time = new_time
        if self.state == UP:
            self.time = time_since_startup - new_time
        if self.state == DOWN:
            self.time = time_since_startup + new_time
    
    def set_state(self, new_state):
        """
        Sets this timer's state to the specified state. The timer's time will
        be modified as needed to fit the new state.
        """
        time = self.get_absolute_time()
        self.state = new_state
        self.set_absolute_time(time)
    
    def on_manual_state_change(self):
        print ("Manual state change on timer " + str(self.number) + 
                " with state set to " + str(self.state))
        # manual_state_change_event(self.number, self.state)
        if self.announce_on_state_change:
            rpc.announce(self.number)
    
    def on_beeping(self):
        print "Timer " + str(self.number) + " is beeping"
        self.is_beeping = True
        self.announce_beeping()
        # state_change_event(self.number, self.state)
        # timer_beeping_event(self.number)
    
    def announce_beeping(self):
        for connection in speak.live_connections:
            def callback(c, result):
                with print_exceptions:
                    if result == 0:
                        c["say_text"]("timer " + str(self.number) + " is beeping", callback=None)
            with print_exceptions:
                connection["get_queue_size"](callback=partial(callback, connection))
    
    def get_time_fields(self):
        """
        Returns a 3-tuple of the hours, minutes, and seconds, in that order,
        of the timer's absolute value.
        """
        value = self.get_absolute_time()
        return value / 3600, value / 60 % 60, value % 60
    
    def build_object_map(self):
        return {"number": self.number, "time": self.time, "state":
                self.state, "name": self.name, "announce_on_state_change":
        self.announce_on_state_change, "announce_count": self.announce_count,
        "announce_interval": self.announce_interval}

@synchronized(lock)
def publish_timer_object():
    timer_object.set_value(build_timer_object())

@synchronized(lock)
def build_timer_object():
    result = {}
    for timer in timer_map.values():
        result[timer.number] = timer.build_object_map()
    return result

class RPC(object):
    """
    A timer server. The timer daemon manages timers and provides information
    about those timers. Timers can be counting up, counting down, or stopped,
    which, when passed to functions, are represented by the integers 1, 2, and
    3, respectively. Timers have a notion of a current time showing on the
    timer. The timer daemon keeps track of each timer's current time and
    increments/decrements it automatically if the timer is counting up or
    counting down, respectively.
    
    Integration with the speak daemon is provided. If any speak servers are
    available on the network when a timer goes off, those speak servers will be
    instructed to say that the timer is beeping.
    
    In order to reduce network overhead, the time of every timer that is not
    currently stopped is stored as an offset from the number of seconds that
    the timer server will have been running when the timer's value is 0. The
    timer server then only has to publish the time since it last started up
    every second, not the new values on every timer. To compute the absolute
    value showing on a timer that is counting up, where time is the timer's
    offset and startup is the current time since the timer server started up,
    you simply compute startup - time. To compute the absolute value showin on a
    timer that is counting down, you compute time - startup. For stopped timers,
    time will already be the absolute value displaying on a timer, not an
    offset.
    
    If you don't want to do the above calculations yourself, you can just call
    get_time(timer_number) on this service every second. That function will
    return the absolute time on the specified timer.
    """
    
    @synchronized(lock)
    def create(self):
        """
        Creates a new timer. This function returns the number of the timer
        just created.
        """
        timer_number = 1
        while timer_number in timer_map:
            timer_number += 1
        timer = Timer(timer_number)
        timer_map[timer_number] = timer
        publish_timer_object()
        return timer_number
    
    @synchronized(lock)
    def set_time(self, timer_number, new_absolute_time):
        """
        Sets the specified timer's absolute time to the specified time.
        """
        timer_map[timer_number].set_absolute_time(new_absolute_time)
        publish_timer_object()
    
    @synchronized(lock)
    def update_time(self, timer_number, amount_to_add):
        """
        Adds the specified amount to the timer's absolute value. The value can
        be negative.
        """
        timer_map[timer_number].set_absolute_time(
                timer_map[timer_number].get_absolute_time() + amount_to_add)
        publish_timer_object()
    
    @synchronized(lock)
    def get_time(self, timer_number):
        """
        Returns the current absolute time of the specified timer.
        """
        return timer_map[timer_number].get_absolute_time()
    
    @synchronized(lock)
    def set(self, timer_number, *args):
        """
        This can be called either as set(timer_number, attribute_map) or as
        set(timer_number, attribute_name, new_value). The former set all of the
        attributes in the specified map on the specified timer to the
        specified values. The latter sets a single attribute.
        
        A quick note: to make it easier to set timer state via autosend, a
        timer's state can be one of "up", "down", or "stopped", which will be
        translated by this function to 1, 2, and 3. Normal programs using
        timerd via the autobus2 module should stick with the numeric constants
        where possible, though, as these names may change. This name
        translation also works for the set_attribute function.
        
        An additional note: "stop" is accepted as a synonym for "stopped" in
        the above translation. I just added support for that.
        """
        if len(args) == 1:
            attributes, = args
            cast(attributes, dict)
        else:
            attributes = {args[0]: args[1]}
        timer = timer_map[timer_number]
        if "announce_interval" in attributes:
            cast(attributes["announce_interval"], int, long)
            timer.announce_interval = attributes["announce_interval"]
        if "name" in attributes:
            cast(attributes["name"], basestring)
            timer.name = attributes["name"]
        if "announce_on_state_change" in attributes:
            cast(attributes["announce_on_state_change"], bool)
            timer.announce_on_state_change = attributes["announce_on_state_change"]
        if "announce_count" in attributes:
            cast(attributes["announce_count"], int, long)
            timer.announce_count = attributes["announce_count"]
        if "state" in attributes:
            if isinstance(attributes["state"], basestring):
                attributes["state"] = {"up": 1, "down": 2, "stop": 3,
                        "stopped": 3}[attributes["state"]]
            if attributes["state"] != timer.state:
                timer.set_state(attributes["state"])
                timer.on_manual_state_change()
                # state_change_event(timer.number, timer.state)
        publish_timer_object()
    
    @synchronized(lock)
    def set_attribute(self, timer_number, name, value):
        """
        Sets a single attribute. This is equivalent to
        set(timer_number, name, value). It was created back when the set
        function could only be called with a map, and generally shouldn't be
        used anymore.
        """
        self.set(timer_number, {name: value})
    
    @synchronized(lock)
    def delete(self, timer_number):
        """
        Deletes the timer with the specified number. An exception will be
        thrown if the specified timer does not exist.
        """
        if timer_number not in timer_map:
            raise Exception("There is no timer with the number " + str(timer_number))
        del timer_map[timer_number]
        publish_timer_object()
    
    @synchronized(lock)
    def announce(self, timer_number):
        """
        Announces the time remaining on the specified timer to all speakd
        servers on the local network. If there aren't any speakd servers
        currently running, this function does nothing.
        """
        timer = timer_map[timer_number]
        field_values = timer.get_time_fields()
        field_names = ["hour", "minute", "second"]
        announce_string = ""
        for field_name, field_value in zip(field_names, field_values):
            if field_value != 0:
                announce_string += (" :n:" + str(field_value) + " :pt:" + 
                        field_name + ":s:" + str(field_value))
        if announce_string == "":
            announce_string = " :n:0 " + field_names[-1] + "s"
        if timer.state == UP:
            state_string = "counting up"
        if timer.state == DOWN:
            state_string = "counting down"
        if timer.state == STOPPED:
            state_string = "stopped"
        announce_string = ("timer :n:" + str(timer_number) + " shows " + 
                announce_string + " " + state_string)
        try:
            speak["say_text"](announce_string, callback=None)
        except:
            pass
    
    @synchronized(lock)
    def dismiss(self, timer_number):
        """
        If the specified timer is currently announcing that it is beeping over
        speakd, this function stops it. Otherwise, this function does nothing.
        """
        timer = timer_map[timer_number]
        if timer.is_beeping:
            timer.is_beeping = False
    
    @synchronized(lock)
    def list_timers(self):
        """
        Returns a list of the numbers of all timers that currently exist on
        this timer daemon.
        """
        return [timer.number for timer in timer_map.values()]
    
    @synchronized(lock)
    def get_attribute(self, timer_number, attribute_name):
        """
        Returns the current value of the specified attribute on the specified
        timer. This is equivalent to get(timer_number, attribute_name), and
        exists for the same reason that set_attribute exists (and hence
        generally shouldn't be used anymore).
        """
        return self.get(timer_number)[attribute_name]
    
    @synchronized(lock)
    def get(self, timer_number, attribute_name=None):
        """
        This function can be invoked as either get(timer_number) or
        get(timer_number, attribute_name).
        
        When invoked as get(timer_number), returns a map of the specified
        timer's current attributes. Not all of
        these can be set with set_attributes or set; the timer's number, for
        example, cannot be changed. This returns the same map that is
        present in the timers object for the specified timer.
        
        When invoked as get(timer_number, attribute_name), returns the value
        of the specified attribute. This would be equivalent to
        get(timer_number)[attribute_name].
        """
        result = timer_map[timer_number].build_object_map()
        if attribute_name is not None:
            return result[attribute_name]
        else:
            return result

last_announce_check = 0

@synchronized(lock)
def run_periodic_actions():
    """
    Runs tasks that should be run once per second.
    """
    global time_since_startup
    global last_announce_check
    now = datetime.now()
    startup_interval = now - startup_time
    time_since_startup = startup_interval.seconds + (startup_interval.days * 86400)
    startup_object.set_value(time_since_startup)
    # print "Startup time: " + str(time_since_startup)
    modified = False
    for timer_number in timer_map:
        timer = timer_map[timer_number]
        time = timer.get_absolute_time()
        if timer.state == DOWN and time <= 0:
            modified = True
            time = 0
            timer.set_absolute_time(0)
            timer.set_state(STOPPED)
            timer.on_beeping()
        if time < 0:
            modified = True
            time = 0
            timer.set_absolute_time(0)
    if modified:
        publish_timer_object()
    if last_announce_check + 10 <= time_since_startup:
        last_announce_check = time_since_startup
        for timer in timer_map.values():
            if timer.is_beeping:
                timer.announce_beeping()

class IntervalThread(Thread):
    """
    A thread that checks the timers every second and performs any actions that
    need to be performed on them.
    """
    def run(self):
        while not is_shut_down:
            sleep(1)
            run_periodic_actions()

def main():
    global is_shut_down
    try:
        with Bus() as bus:
            run(bus)
    except KeyboardInterrupt:
        print "Interrupted, shutting down"
    finally:
        is_shut_down = True

def run(bus_to_use):
    global bus
    global speak
    global rpc
    global timer_object
    global startup_object
    global timer_beeping_event
    global state_change_event
    global manual_state_change_event
    bus = bus_to_use
    speak = bus.get_service_proxy({"type": "speak"}, multiple=True)
    rpc = RPC()
    service = bus.create_service({"type": "timer"}, from_py_object=rpc, active=False)
    timer_object = service.create_object("timers", {}, "This object is a map of "
            "maps. The outer map maps timer numbers to maps representing "
            "those timers. Each timer's map contains a number of keys which "
            "will be described at some point (Alex: TODO: write this), which "
            "contain the timer's info.")
    startup_object = service.create_object("startup", 0, "The number of seconds since "
            "timerd started up.")
#    timer_beeping_event = bus.add_event("timer", "beeping", "This event is "
#            "fired whenever a timer that is counting down reaches zero and "
#            "has its state reset to stopped. It is passed one parameter, "
#            "the number of the timer that just went off. The event is fired "
#            "before the timer object is updated to reflect that the timer is "
#            "now stopped.")
#    state_change_event = bus.add_event("timer", "state_change", "This event "
#            "is fired whenever a timer changes state, both when a user "
#            "manually changes the timer's state and when the state changes "
#            "due to the timer reaching zero. This event is passed two "
#            "parameters: the number of the timer whose state changed and the "
#            "new state of the timer.")
#    manual_state_change_event = bus.add_event("timer", "manual_state_change",
#            "This event is fired whenever a timer's state is changed "
#            "manually by a call to either the set or set_attribute functions. "
#            "This event is passed the same set of parameters that "
#            "state_change is passed.")
    service.activate()
    IntervalThread().start()
    wait_for_interrupt()



























"""
This module contains all of the decorators that can be used to specify the
types of parameters given to Autobus functions, events, and so on.

Decorating functions intended to be given to Autobus with type information
provides two useful things: the actual arguments to the function, when it is
called, will be type checked (thus guaranteeing that an object of the wrong
type won't be passed in on accident), and remote clients can ask a function for
the types of its arguments.
"""